# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-
# pylint: disable=too-many-lines

import abc

from cryptodatahub.tls.algorithm import TlsCipherSuite, SslCipherKind

from cryptoparser.common.base import TwoByteEnumParsable, ThreeByteEnumParsable


class TlsCipherSuiteFactory(TwoByteEnumParsable):
    @classmethod
    def get_enum_class(cls):
        return TlsCipherSuite

    @abc.abstractmethod
    def compose(self):
        raise NotImplementedError()


class SslCipherKindFactory(ThreeByteEnumParsable):
    @classmethod
    def get_enum_class(cls):
        return SslCipherKind

    @abc.abstractmethod
    def compose(self):
        raise NotImplementedError()



# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-

import unittest


from cryptodatahub.common.exception import InvalidValue
from cryptodatahub.common.grade import Grade

from cryptoparser.common.exception import NotEnoughData

from cryptoparser.tls.version import TlsVersion, TlsProtocolVersion


class TestTlsProtocolVersion(unittest.TestCase):
    def test_parse(self):
        parsable = b'\x03\xff'
        expected_error_message = ' is not a valid TlsVersionFactory'
        with self.assertRaisesRegex(InvalidValue, expected_error_message):
            # pylint: disable=expression-not-assigned
            TlsProtocolVersion.parse_exact_size(parsable)

        expected_error_message = ' is not a valid TlsVersionFactory'
        with self.assertRaisesRegex(InvalidValue, expected_error_message):
            # pylint: disable=expression-not-assigned
            TlsProtocolVersion.parse_exact_size(b'\x8f\x00')

        with self.assertRaises(NotEnoughData) as context_manager:
            TlsProtocolVersion.parse_exact_size(b'\xff')
        self.assertEqual(context_manager.exception.bytes_needed, 1)

    def test_compose(self):
        self.assertEqual(
            b'\x03\x03',
            TlsProtocolVersion(TlsVersion.TLS1_2).compose()
        )

        self.assertEqual(
            b'\x7f\x12',
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_18).compose()
        )

    def test_lt(self):
        self.assertLess(
            TlsProtocolVersion(TlsVersion.SSL2),
            TlsProtocolVersion(TlsVersion.SSL3)
        )
        self.assertGreater(
            TlsProtocolVersion(TlsVersion.SSL3),
            TlsProtocolVersion(TlsVersion.SSL2)
        )

        self.assertLess(
            TlsProtocolVersion(TlsVersion.SSL3),
            TlsProtocolVersion(TlsVersion.TLS1)
        )
        self.assertGreater(
            TlsProtocolVersion(TlsVersion.TLS1),
            TlsProtocolVersion(TlsVersion.SSL3)
        )

        self.assertLess(
            TlsProtocolVersion(TlsVersion.TLS1_1),
            TlsProtocolVersion(TlsVersion.TLS1_2)
        )
        self.assertGreater(
            TlsProtocolVersion(TlsVersion.TLS1_2),
            TlsProtocolVersion(TlsVersion.TLS1_1)
        )

        self.assertLess(
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_1),
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_2)
        )
        self.assertGreater(
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_2),
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_1)
        )

        self.assertLess(
            TlsProtocolVersion(TlsVersion.TLS1_2),
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_0)
        )
        self.assertGreater(
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_0),
            TlsProtocolVersion(TlsVersion.TLS1_2)
        )

        self.assertLess(
            TlsProtocolVersion(TlsVersion.TLS1_2),
            TlsProtocolVersion(TlsVersion.TLS1_3)
        )
        self.assertGreater(
            TlsProtocolVersion(TlsVersion.TLS1_3),
            TlsProtocolVersion(TlsVersion.TLS1_2)
        )

        self.assertLess(
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_28),
            TlsProtocolVersion(TlsVersion.TLS1_3)
        )
        self.assertGreater(
            TlsProtocolVersion(TlsVersion.TLS1_3),
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_28)
        )

        self.assertLess(
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_0),
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_28)
        )
        self.assertGreater(
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_28),
            TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_0)
        )

    def test_set(self):
        self.assertEqual(
            2,
            len(set([
                TlsProtocolVersion(TlsVersion.TLS1_1),
                TlsProtocolVersion(TlsVersion.TLS1_2)
            ]))
        )
        self.assertEqual(
            1,
            len(set([
                TlsProtocolVersion(TlsVersion.TLS1_1),
                TlsProtocolVersion(TlsVersion.TLS1_1)
            ]))
        )

        self.assertEqual(
            2,
            len(set([
                TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_1),
                TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_2)
            ]))
        )
        self.assertEqual(
            1,
            len(set([
                TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_1),
                TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_1)
            ]))
        )

    def test_as_json(self):
        self.assertEqual(TlsProtocolVersion(TlsVersion.SSL3).as_json(), '\"ssl3\"')
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1).as_json(), '\"tls1\"')
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1_2).as_json(), '\"tls1_2\"')
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_24).as_json(), '\"tls1_3_draft_24\"')
        self.assertEqual(
            TlsProtocolVersion(TlsVersion.TLS1_3_GOOGLE_EXPERIMENT_2).as_json(),
            '\"tls1_3_google_experiment_2\"'
        )

    def test_as_markdown(self):
        self.assertEqual(TlsProtocolVersion(TlsVersion.SSL3).as_markdown(), 'SSL 3.0')
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1).as_markdown(), 'TLS 1.0')
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1_2).as_markdown(), 'TLS 1.2')
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_24).as_markdown(), 'TLS 1.3 Draft 24')
        self.assertEqual(
            TlsProtocolVersion(TlsVersion.TLS1_3_GOOGLE_EXPERIMENT_2).as_markdown(),
            'TLS 1.3 Google Experiment 2'
        )

    def test_str(self):
        self.assertEqual(str(TlsProtocolVersion(TlsVersion.SSL2)), 'SSL 2.0')
        self.assertEqual(str(TlsProtocolVersion(TlsVersion.SSL3)), 'SSL 3.0')
        self.assertEqual(str(TlsProtocolVersion(TlsVersion.TLS1)), 'TLS 1.0')
        self.assertEqual(str(TlsProtocolVersion(TlsVersion.TLS1_2)), 'TLS 1.2')
        self.assertEqual(str(TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_24)), 'TLS 1.3 Draft 24')
        self.assertEqual(str(TlsProtocolVersion(TlsVersion.TLS1_3_GOOGLE_EXPERIMENT_2)), 'TLS 1.3 Google Experiment 2')

    def test_grade(self):
        self.assertEqual(TlsProtocolVersion(TlsVersion.SSL2).grade, Grade.INSECURE)
        self.assertEqual(TlsProtocolVersion(TlsVersion.SSL3).grade, Grade.INSECURE)
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1).grade, Grade.DEPRECATED)
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1_1).grade, Grade.DEPRECATED)
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1_3_DRAFT_28).grade, Grade.DEPRECATED)
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1_2).grade, Grade.SECURE)
        self.assertEqual(TlsProtocolVersion(TlsVersion.TLS1_3).grade, Grade.SECURE)



# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-

from test.common.classes import TestClasses

from cryptodatahub.common.key import PublicKeyX509Base
from cryptodatahub.common.entity import Entity

from cryptoparser.common.x509 import SignedCertificateTimestampList


class TestTlsPubKeys(TestClasses.TestKeyBase):
    def test_signed_certificate_timestamps(self):
        certificate = self._get_public_key_x509('rsa8192.badssl.com_root_ca.crt')
        self.assertEqual(certificate.signed_certificate_timestamps, SignedCertificateTimestampList([]))

        certificate = self._get_public_key_x509('rsa8192.badssl.com_certificate.crt')
        self.assertIn(
            Entity.GOOGLE,
            [sct.log.operator for sct in certificate.signed_certificate_timestamps]
        )

    def test_asdict(self):
        certificate = self._get_public_key_x509('rsa8192.badssl.com_certificate.crt')
        dict_result = certificate._asdict()
        self.assertIn(
            Entity.GOOGLE,
            [sct.log.operator for sct in dict_result.pop('signed_certificate_timestamps')],
        )

        self.assertEqual(PublicKeyX509Base._asdict(certificate), dict_result)



# SPDX-License-Identifier: MPL-2.0
"""ISAKMP version handling."""

import abc
import enum

import attr

from cryptodatahub.common.grade import GradeableSimple, Grade

from cryptoparser.common.parse import ParsableBase, ParserBinary, ComposerBinary
from cryptoparser.common.exception import NotEnoughData, InvalidType
from cryptoparser.common.base import OneByteEnumParsable


class IsakmpVersion(enum.IntEnum):
    """ISAKMP version."""
    V1 = 0x01  # ISAKMP v1
    V2 = 0x02  # ISAKMP v2 (IKEv2)

    @property
    def identifier(self):
        """Get version identifier."""
        return f"ikev{self.value}"


class IsakmpVersionFactory(OneByteEnumParsable):
    """ISAKMP version."""
    @classmethod
    def get_enum_class(cls):
        return IsakmpVersion

    @abc.abstractmethod
    def compose(self):
        raise NotImplementedError()


@attr.s(frozen=True)
class IsakmpProtocolVersion(ParsableBase, GradeableSimple):
    """ISAKMP protocol version parser."""
    HEADER_SIZE = 1

    major: IsakmpVersion = attr.ib(validator=attr.validators.instance_of(IsakmpVersion))
    minor: int = attr.ib(validator=attr.validators.instance_of(int))

    @property
    def grade(self):
        if self.major == IsakmpVersion.V1:
            return Grade.DEPRECATED

        return Grade.SECURE

    def __str__(self):
        return f"IKEv{self.major.value} ({self.minor})"

    @classmethod
    def _parse(cls, parsable):
        if len(parsable) < cls.HEADER_SIZE:
            raise NotEnoughData(cls.HEADER_SIZE - len(parsable))

        parser = ParserBinary(parsable)
        parser.parse_numeric('version', 1)

        major_version = (parser['version'] >> 4) & 0x0f
        minor_version = parser['version'] & 0x0f

        if major_version not in [v.value for v in IsakmpVersion]:
            raise InvalidType()

        return cls(
            major=IsakmpVersion(major_version),
            minor=minor_version,
        ), parser.parsed_length

    def compose(self):
        composer = ComposerBinary()

        version = (self.major.value << 4) | self.minor
        composer.compose_numeric(version, 1)

        return composer.composed_bytes

    @property
    def version(self):
        """Get version as string."""
        return f"{self.major.value}.{self.minor}"



# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-

import collections
import unittest

from cryptodatahub.ike.algorithm import (
    Ikev2DiffieHellmanGroup,
    Ikev2NotifyType,
    Ikev2ProtocolId,
    Ikev2TransformAttributeType,
)

from cryptoparser.common.exception import NotEnoughData, TooMuchData
from cryptoparser.ike.ikev2 import (
    Ikev2NotifyPayloadInvalidKe,
    Ikev2PayloadCertificateRequest,
    Ikev2PayloadDelete,
    Ikev2PayloadFlags,
    Ikev2PayloadKeyExchange,
    Ikev2PayloadNonce,
    Ikev2PayloadNotifyAuthenticationFailed,
    Ikev2PayloadNotifyUnparsed,
    Ikev2PayloadType,
    Ikev2PayloadVendorId,
    TransformAttributeKeyLength,

)

from .classes import Ikev2PayloadBaseTest, Ikev2PayloadNotifyNoDataTest


class TestIkev2PayloadBase(unittest.TestCase):
    def setUp(self):
        self.test_data = b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
        self.test_payload_minimal = Ikev2PayloadBaseTest(
            flags=set(),
            test_data=b'',
        )
        self.test_payload_minimal.next_payload = Ikev2PayloadType.NONE

        self.test_dict_minimal = collections.OrderedDict([
            ('next_payload', b'\x00'),
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x04'),
            ('test_data', b''),
        ])
        self.test_bytes_minimal = b''.join(self.test_dict_minimal.values())

        self.test_payload_with_data = Ikev2PayloadBaseTest(
            flags={Ikev2PayloadFlags.CRITICAL},
            test_data=self.test_data
        )
        self.test_payload_with_data.next_payload = Ikev2PayloadType.NONE

        self.test_dict_with_data = collections.OrderedDict([
            ('next_payload', b'\x00'),
            ('flags', b'\x80'),
            ('payload_length', b'\x00\x14'),
            ('test_data', self.test_data),
        ])
        self.test_bytes_with_data = b''.join(self.test_dict_with_data.values())

    def test_parse(self):
        parsed_payload = Ikev2PayloadBaseTest.parse_exact_size(self.test_bytes_minimal)
        self.assertEqual(parsed_payload.flags, self.test_payload_minimal.flags)
        self.assertEqual(parsed_payload.test_data, self.test_payload_minimal.test_data)
        self.assertEqual(parsed_payload.next_payload, self.test_payload_minimal.next_payload)

        parsed_payload = Ikev2PayloadBaseTest.parse_exact_size(self.test_bytes_with_data)
        self.assertEqual(parsed_payload.flags, self.test_payload_with_data.flags)
        self.assertEqual(parsed_payload.test_data, self.test_payload_with_data.test_data)
        self.assertEqual(parsed_payload.next_payload, self.test_payload_with_data.next_payload)

    def test_compose(self):
        self.assertEqual(self.test_payload_minimal.compose(), self.test_bytes_minimal)
        self.assertEqual(self.test_payload_with_data.compose(), self.test_bytes_with_data)

    def test_error_parse_not_enough_data(self):
        with self.assertRaises(NotEnoughData) as context_manager:
            Ikev2PayloadBaseTest.parse_exact_size(b'\x00')
        self.assertEqual(context_manager.exception.bytes_needed, 3)

    def test_error_payload_validation(self):
        with self.assertRaises(TypeError) as context_manager:
            Ikev2PayloadBaseTest(
                flags={'invalid_flag'},
                test_data=self.test_data
            )
        exception_str = str(context_manager.exception)
        self.assertIn("flags", exception_str)
        self.assertIn("must be", exception_str)
        self.assertIn("Ikev2PayloadFlags", exception_str)

    def test_next_payload(self):
        payload = Ikev2PayloadBaseTest(
            flags={Ikev2PayloadFlags.CRITICAL},
            test_data=self.test_data
        )
        payload.next_payload = Ikev2PayloadType.KE

        composed = payload.compose()
        self.assertEqual(composed[0], Ikev2PayloadType.KE.value.code)

        parsed, _ = Ikev2PayloadBaseTest._parse(composed)  # pylint: disable=protected-access
        self.assertEqual(parsed.next_payload, Ikev2PayloadType.KE)


class TestIkev2PayloadKeyExchange(unittest.TestCase):
    def setUp(self):
        self.dh_group = Ikev2DiffieHellmanGroup.MODP_GROUP_2048_BIT
        self.key_exchange_data = b'\x00\x01\x02\x03\x04\x05\x06\x07'
        self.key_exchange_payload = Ikev2PayloadKeyExchange(
            flags=set(),
            dh_group=self.dh_group,
            key_exchange_data=self.key_exchange_data
        )
        self.key_exchange_payload.next_payload = Ikev2PayloadType.NONE

        self.key_exchange_dict = collections.OrderedDict([
            ('next_payload', b'\x00'),
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x10'),
            ('dh_group', b'\x00\x0e'),
            ('reserved2', b'\x00\x00'),
            ('key_exchange_data', self.key_exchange_data),
        ])
        self.key_exchange_bytes = b''.join(self.key_exchange_dict.values())

    def test_get_payload_type(self):
        self.assertEqual(Ikev2PayloadKeyExchange.get_payload_type(), Ikev2PayloadType.KE)

    def test_parse(self):
        parsed_ke = Ikev2PayloadKeyExchange.parse_exact_size(self.key_exchange_bytes)
        self.assertEqual(parsed_ke.dh_group, self.dh_group)
        self.assertEqual(parsed_ke.key_exchange_data, self.key_exchange_data)

    def test_compose(self):
        composed_bytes = self.key_exchange_payload.compose()
        self.assertGreater(len(composed_bytes), Ikev2PayloadKeyExchange.HEADER_SIZE)

        parsed_ke = Ikev2PayloadKeyExchange.parse_exact_size(composed_bytes)
        self.assertEqual(parsed_ke.key_exchange_data, self.key_exchange_payload.key_exchange_data)

    def test_round_trip(self):
        composed_bytes = self.key_exchange_payload.compose()
        parsed_payload: Ikev2PayloadKeyExchange = Ikev2PayloadKeyExchange.parse_exact_size(composed_bytes)

        self.assertEqual(parsed_payload.dh_group, self.key_exchange_payload.dh_group)
        self.assertEqual(parsed_payload.key_exchange_data, self.key_exchange_payload.key_exchange_data)
        self.assertEqual(parsed_payload.flags, self.key_exchange_payload.flags)
        self.assertEqual(parsed_payload.next_payload, self.key_exchange_payload.next_payload)


class TestIkev2PayloadNonce(unittest.TestCase):  # pylint: disable=too-many-instance-attributes
    def setUp(self):
        self.nonce_data = b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
        self.nonce_payload = Ikev2PayloadNonce(
            flags=set(),
            nonce_data=self.nonce_data
        )
        self.nonce_payload.next_payload = Ikev2PayloadType.NONE

        self.nonce_dict = collections.OrderedDict([
            ('next_payload', b'\x00'),
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x14'),
            ('nonce_data', self.nonce_data),
        ])
        self.nonce_bytes = b''.join(self.nonce_dict.values())

        self.nonce_dict_too_small = collections.OrderedDict([
            ('next_payload', b'\x00'),
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x13'),
            ('nonce_data', b'\x00' * 15),
        ])
        self.nonce_bytes_too_small = b''.join(self.nonce_dict_too_small.values())

        self.nonce_dict_too_large = collections.OrderedDict([
            ('next_payload', b'\x00'),
            ('flags', b'\x00'),
            ('payload_length', b'\x01\x05'),
            ('nonce_data', b'\x00' * 257),
        ])
        self.nonce_bytes_too_large = b''.join(self.nonce_dict_too_large.values())

    def test_get_payload_type(self):
        self.assertEqual(Ikev2PayloadNonce.get_payload_type(), Ikev2PayloadType.NONCE)

    def test_parse(self):
        parsed_nonce = Ikev2PayloadNonce.parse_exact_size(self.nonce_bytes)
        self.assertEqual(parsed_nonce.nonce_data, self.nonce_data)
        self.assertEqual(parsed_nonce.next_payload, Ikev2PayloadType.NONE)
        self.assertEqual(parsed_nonce.flags, set())

    def test_compose(self):
        composed_bytes = self.nonce_payload.compose()
        self.assertIsInstance(composed_bytes, (bytes, bytearray))
        self.assertGreater(len(composed_bytes), Ikev2PayloadNonce.HEADER_SIZE)

        parsed_nonce = Ikev2PayloadNonce.parse_exact_size(composed_bytes)
        self.assertEqual(parsed_nonce.nonce_data, self.nonce_payload.nonce_data)
        self.assertEqual(parsed_nonce.next_payload, self.nonce_payload.next_payload)
        self.assertEqual(parsed_nonce.flags, self.nonce_payload.flags)

    def test_error_invalid_nonce_data(self):
        with self.assertRaises(Exception) as context_manager:
            Ikev2PayloadNonce(
                flags=set(),
                nonce_data="not_bytes"
            )
        self.assertTrue(len(str(context_manager.exception)) > 0)

    def test_minimal_nonce_data(self):
        nonce_empty = Ikev2PayloadNonce(
            flags=set(),
            nonce_data=b'\x00' * 16
        )
        nonce_empty.next_payload = Ikev2PayloadType.NONE
        composed_bytes = nonce_empty.compose()
        parsed_nonce = Ikev2PayloadNonce.parse_exact_size(composed_bytes)
        self.assertEqual(parsed_nonce.nonce_data, b'\x00' * 16)

    def test_too_small_nonce_data(self):
        with self.assertRaises(NotEnoughData) as context_manager:
            Ikev2PayloadNonce.parse_exact_size(self.nonce_bytes_too_small)
        self.assertEqual(context_manager.exception.bytes_needed, 16 - 15)

    def test_too_large_nonce_data(self):
        with self.assertRaises(TooMuchData) as context_manager:
            Ikev2PayloadNonce.parse_exact_size(self.nonce_bytes_too_large)
        self.assertEqual(context_manager.exception.bytes_needed, 257 - 256)

    def test_error_nonce_data_validation_min_length(self):
        with self.assertRaises(ValueError):
            Ikev2PayloadNonce(flags=set(), nonce_data=b'\x01\x02\x03\x04\x05\x06\x07\x08')

    def test_error_nonce_data_validation_max_length(self):
        with self.assertRaises(ValueError):
            Ikev2PayloadNonce(flags=set(), nonce_data=b'\x01' * 257)

    def test_round_trip(self):
        composed_bytes = self.nonce_payload.compose()
        parsed_payload: Ikev2PayloadNonce = Ikev2PayloadNonce.parse_exact_size(composed_bytes)

        self.assertEqual(parsed_payload.nonce_data, self.nonce_payload.nonce_data)
        self.assertEqual(parsed_payload.flags, self.nonce_payload.flags)
        self.assertEqual(parsed_payload.next_payload, self.nonce_payload.next_payload)


class TestIkev2PayloadDelete(unittest.TestCase):  # pylint: disable=too-many-instance-attributes
    def setUp(self):
        self.protocol_id = Ikev2ProtocolId.IKE
        self.spis = [0x1234567890abcdef, 0xfedcba0987654321]
        self.delete_payload = Ikev2PayloadDelete(
            flags=set(),
            protocol_id=self.protocol_id,
            spis=self.spis
        )
        self.delete_payload.next_payload = Ikev2PayloadType.NONE

        self.delete_dict = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x18'),  # 24 bytes (header + protocol_id + spi_size + num_spis + spis)
            ('protocol_id', b'\x01'),  # IKE protocol
            ('spi_size', b'\x10'),  # 16 bytes (2 SPIs * 8 bytes each)
            ('num_spis', b'\x00\x02'),  # 2 SPIs
            ('spis', b'\x12\x34\x56\x78\x90\xab\xcd\xef\xfe\xdc\xba\x09\x87\x65\x43\x21'),
        ])
        self.delete_bytes = b''.join(self.delete_dict.values())

        self.empty_delete_payload = Ikev2PayloadDelete(
            flags={Ikev2PayloadFlags.CRITICAL},
            protocol_id=self.protocol_id,
            spis=[]
        )
        self.empty_delete_payload.next_payload = Ikev2PayloadType.NONE

        self.empty_delete_dict = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x80'),  # CRITICAL (0x80)
            ('payload_length', b'\x00\x08'),  # 8 bytes (header + protocol_id + spi_size + num_spis)
            ('protocol_id', b'\x01'),  # IKE protocol
            ('spi_size', b'\x00'),
            ('num_spis', b'\x00\x00'),
        ])
        self.empty_delete_bytes = b''.join(self.empty_delete_dict.values())

    def test_get_payload_type(self):
        self.assertEqual(Ikev2PayloadDelete.get_payload_type(), Ikev2PayloadType.DELETE)

    def test_parse(self):
        parsed_delete = Ikev2PayloadDelete.parse_exact_size(self.delete_bytes)
        self.assertEqual(parsed_delete.protocol_id, self.protocol_id)
        self.assertEqual(parsed_delete.spis, self.spis)
        self.assertEqual(parsed_delete.next_payload, Ikev2PayloadType.NONE)
        self.assertEqual(parsed_delete.flags, set())

    def test_compose(self):
        composed_bytes = self.delete_payload.compose()
        self.assertIsInstance(composed_bytes, (bytes, bytearray))
        self.assertGreater(len(composed_bytes), Ikev2PayloadDelete.HEADER_SIZE)

        parsed_delete = Ikev2PayloadDelete.parse_exact_size(composed_bytes)
        self.assertEqual(parsed_delete.protocol_id, self.delete_payload.protocol_id)
        self.assertEqual(parsed_delete.spis, self.delete_payload.spis)
        self.assertEqual(parsed_delete.next_payload, self.delete_payload.next_payload)
        self.assertEqual(parsed_delete.flags, self.delete_payload.flags)

    def test_empty_spis(self):
        composed_bytes = self.empty_delete_payload.compose()
        parsed_delete = Ikev2PayloadDelete.parse_exact_size(composed_bytes)
        self.assertEqual(parsed_delete.spis, [])
        self.assertEqual(parsed_delete.protocol_id, self.protocol_id)
        self.assertEqual(parsed_delete.flags, {Ikev2PayloadFlags.CRITICAL})

    def test_error_parse_not_enough_data(self):
        with self.assertRaises(NotEnoughData) as context_manager:
            Ikev2PayloadDelete.parse_exact_size(b'\x00\x01\x02')
        self.assertEqual(context_manager.exception.bytes_needed, 1)

    def test_error_invalid_protocol_id(self):
        with self.assertRaises(TypeError):
            Ikev2PayloadDelete(
                flags=set(),
                protocol_id="invalid_protocol",
                spis=self.spis
            )

    def test_error_invalid_spis_type(self):
        with self.assertRaises(TypeError):
            Ikev2PayloadDelete(
                flags=set(),
                protocol_id=self.protocol_id,
                spis=["not_an_int"]
            )

    def test_round_trip(self):
        composed_bytes = self.delete_payload.compose()
        parsed_payload: Ikev2PayloadDelete = Ikev2PayloadDelete.parse_exact_size(composed_bytes)

        self.assertEqual(parsed_payload.protocol_id, self.delete_payload.protocol_id)
        self.assertEqual(parsed_payload.spis, self.delete_payload.spis)
        self.assertEqual(parsed_payload.flags, self.delete_payload.flags)
        self.assertEqual(parsed_payload.next_payload, self.delete_payload.next_payload)


class TestIkev2PayloadNotifyNoData(unittest.TestCase):  # pylint: disable=too-many-instance-attributes
    def setUp(self):
        self.protocol_id = Ikev2ProtocolId.IKE
        self.notify_type = Ikev2NotifyType.AUTHENTICATION_FAILED
        self.spi = b'\x00\x01\x02\x03\x04\x05\x06\x07'

        self.notify_payload_minimal = Ikev2PayloadNotifyNoDataTest(
            flags=set(),
            protocol_id=self.protocol_id,
            notify_type=self.notify_type,
            spi=b''
        )
        self.notify_payload_minimal.next_payload = Ikev2PayloadType.NONE

        self.notify_dict_minimal = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x08'),  # 8 bytes (header + protocol_id + spi_size + notify_type)
            ('protocol_id', b'\x01'),  # IKE protocol
            ('spi_size', b'\x00'),
            ('notify_type', b'\x00\x18'),  # AUTHENTICATION_FAILED (0x0018)
        ])
        self.notify_bytes_minimal = b''.join(self.notify_dict_minimal.values())

        self.notify_payload_with_spi = Ikev2PayloadNotifyNoDataTest(
            flags={Ikev2PayloadFlags.CRITICAL},
            protocol_id=self.protocol_id,
            notify_type=self.notify_type,
            spi=self.spi
        )
        self.notify_payload_with_spi.next_payload = Ikev2PayloadType.NONE

        self.notify_dict_with_spi = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x80'),  # CRITICAL (0x80)
            ('payload_length', b'\x00\x10'),  # 16 bytes (header + protocol_id + spi_size + notify_type + spi)
            ('protocol_id', b'\x01'),  # IKE protocol
            ('spi_size', b'\x08'),
            ('notify_type', b'\x00\x18'),  # AUTHENTICATION_FAILED (0x0018)
            ('spi', self.spi),
        ])
        self.notify_bytes_with_spi = b''.join(self.notify_dict_with_spi.values())

    def test_parse(self):
        parsed_notify = Ikev2PayloadNotifyNoDataTest.parse_exact_size(self.notify_bytes_minimal)
        self.assertEqual(parsed_notify.protocol_id, self.protocol_id)
        self.assertEqual(parsed_notify.type, self.notify_type)
        self.assertEqual(parsed_notify.spi, b'')
        self.assertEqual(parsed_notify.next_payload, Ikev2PayloadType.NONE)
        self.assertEqual(parsed_notify.flags, set())

        parsed_notify = Ikev2PayloadNotifyNoDataTest.parse_exact_size(self.notify_bytes_with_spi)
        self.assertEqual(parsed_notify.protocol_id, self.protocol_id)
        self.assertEqual(parsed_notify.type, self.notify_type)
        self.assertEqual(parsed_notify.spi, self.spi)
        self.assertEqual(parsed_notify.next_payload, Ikev2PayloadType.NONE)
        self.assertEqual(parsed_notify.flags, {Ikev2PayloadFlags.CRITICAL})

    def test_compose(self):
        self.assertEqual(self.notify_payload_minimal.compose(), self.notify_bytes_minimal)
        self.assertEqual(self.notify_payload_with_spi.compose(), self.notify_bytes_with_spi)

    def test_error_parse_not_enough_data(self):
        with self.assertRaises(NotEnoughData) as context_manager:
            Ikev2PayloadNotifyNoDataTest.parse_exact_size(b'\x00\x01\x02')
        self.assertEqual(context_manager.exception.bytes_needed, 1)

    def test_error_payload_validation(self):
        with self.assertRaises(TypeError) as context_manager:
            Ikev2PayloadNotifyNoDataTest(
                flags={'invalid_flag'},
                protocol_id=self.protocol_id,
                notify_type=self.notify_type,
                spi=self.spi
            )
        exception_str = str(context_manager.exception)
        self.assertIn("flags", exception_str)
        self.assertIn("must be", exception_str)
        self.assertIn("Ikev2PayloadFlags", exception_str)

    def test_error_invalid_protocol_id(self):
        with self.assertRaises(TypeError):
            Ikev2PayloadNotifyNoDataTest(
                flags=set(),
                protocol_id="invalid_protocol",
                notify_type=self.notify_type,
                spi=self.spi
            )

    def test_error_invalid_notify_type(self):
        with self.assertRaises(TypeError):
            Ikev2PayloadNotifyNoDataTest(
                flags=set(),
                protocol_id=self.protocol_id,
                notify_type="invalid_type",
                spi=self.spi
            )

    def test_error_invalid_spi(self):
        # The spi attribute has converter=bytes, so test with something that can't be converted
        with self.assertRaises((TypeError, ValueError)):
            Ikev2PayloadNotifyNoDataTest(
                flags=set(),
                protocol_id=self.protocol_id,
                notify_type=self.notify_type,
                spi=None
            )


class TestIkev2PayloadNotifyAuthenticationFailed(unittest.TestCase):
    def setUp(self):
        self.protocol_id = Ikev2ProtocolId.IKE

        self.notify_dict_authentication_failed = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x08'),  # 8 bytes (header + protocol_id + spi_size + notify_type)
            ('protocol_id', b'\x01'),  # IKE protocol
            ('spi_size', b'\x00'),
            ('notify_type', b'\x00\x18'),  # AUTHENTICATION_FAILED (0x0018)
        ])
        self.notify_bytes_authentication_failed = b''.join(self.notify_dict_authentication_failed.values())

    def test_get_message_type(self):
        self.assertEqual(Ikev2PayloadNotifyAuthenticationFailed._get_message_type(),  # pylint: disable=protected-access
                         Ikev2NotifyType.AUTHENTICATION_FAILED)

    def test_parse_authentication_failed(self):
        parsed_notify = Ikev2PayloadNotifyAuthenticationFailed.parse_exact_size(
            self.notify_bytes_authentication_failed)
        self.assertEqual(parsed_notify.type, Ikev2NotifyType.AUTHENTICATION_FAILED)
        self.assertEqual(parsed_notify.protocol_id, self.protocol_id)


class TestIkev2PayloadNotifyUnparsed(unittest.TestCase):
    def setUp(self):
        self.notify_type = Ikev2NotifyType.INVALID_SYNTAX
        self.notify_data = (
            b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
            b'\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
        )

        self.notify_dict_with_data = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x28'),  # 40 bytes (8 header + 32 data)
            ('protocol_id', b'\x01'),  # IKE protocol
            ('spi_size', b'\x00'),
            ('notify_type', b'\x00\x07'),  # INVALID_SYNTAX (0x0007)
        ])
        self.notify_bytes_with_data = b''.join(self.notify_dict_with_data.values()) + self.notify_data

    def test_parse(self):
        parsed_notify: Ikev2PayloadNotifyUnparsed = Ikev2PayloadNotifyUnparsed.parse_exact_size(
            self.notify_bytes_with_data)
        self.assertEqual(parsed_notify.data, self.notify_data)  # pylint: disable=no-member

    def test_compose(self):
        notify_payload = Ikev2PayloadNotifyUnparsed(
            flags=set(),
            protocol_id=Ikev2ProtocolId.IKE,
            type=self.notify_type,
            spi=b'',
            data=self.notify_data
        )
        notify_payload.next_payload = Ikev2PayloadType.NONE

        composed_bytes = notify_payload.compose()
        self.assertEqual(composed_bytes, self.notify_bytes_with_data)

    def test_round_trip(self):
        original_payload = Ikev2PayloadNotifyUnparsed(
            flags=set(),
            protocol_id=Ikev2ProtocolId.IKE,
            type=self.notify_type,
            spi=b'',
            data=self.notify_data
        )
        original_payload.next_payload = Ikev2PayloadType.NONE

        composed_bytes = original_payload.compose()
        parsed_payload: Ikev2PayloadNotifyUnparsed = Ikev2PayloadNotifyUnparsed.parse_exact_size(composed_bytes)

        self.assertEqual(parsed_payload.data, original_payload.data)  # pylint: disable=no-member


class TestIkev2NotifyPayloadInvalidKe(unittest.TestCase):
    def setUp(self):
        self.dh_group = Ikev2DiffieHellmanGroup.MODP_GROUP_2048_BIT

        self.invalid_ke_dict = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x0a'),  # 10 bytes (8 header + 2 dh_group)
            ('protocol_id', b'\x01'),  # IKE protocol
            ('spi_size', b'\x00'),
            ('notify_type', b'\x00\x11'),  # INVALID_KE_PAYLOAD (0x0011)
            ('dh_group', b'\x00\x0e'),  # MODP_GROUP_2048_BIT (0x000e)
        ])
        self.invalid_ke_bytes = b''.join(self.invalid_ke_dict.values())
        self.invalid_ke_payload = Ikev2NotifyPayloadInvalidKe(
            flags=set(),
            protocol_id=Ikev2ProtocolId.IKE,
            type=Ikev2NotifyType.INVALID_KE_PAYLOAD,
            spi=b'',
            dh_group=self.dh_group
        )
        self.invalid_ke_payload.next_payload = Ikev2PayloadType.NONE

    def test_get_message_type(self):
        self.assertEqual(Ikev2NotifyPayloadInvalidKe._get_message_type(),  # pylint: disable=protected-access
                         Ikev2NotifyType.INVALID_KE_PAYLOAD)

    def test_parse(self):
        parsed_notify: Ikev2NotifyPayloadInvalidKe = Ikev2NotifyPayloadInvalidKe.parse_exact_size(self.invalid_ke_bytes)
        self.assertEqual(parsed_notify.type, Ikev2NotifyType.INVALID_KE_PAYLOAD)
        self.assertEqual(parsed_notify.protocol_id, Ikev2ProtocolId.IKE)
        self.assertEqual(parsed_notify.dh_group, self.dh_group)  # pylint: disable=no-member

    def test_compose(self):
        composed_bytes = self.invalid_ke_payload.compose()
        self.assertEqual(composed_bytes, self.invalid_ke_bytes)

    def test_round_trip(self):
        composed_bytes = self.invalid_ke_payload.compose()
        parsed_payload: Ikev2NotifyPayloadInvalidKe = Ikev2NotifyPayloadInvalidKe.parse_exact_size(composed_bytes)

        # Verify all attributes are preserved
        self.assertEqual(parsed_payload.dh_group, self.invalid_ke_payload.dh_group)  # pylint: disable=no-member
        self.assertEqual(parsed_payload.flags, self.invalid_ke_payload.flags)
        self.assertEqual(parsed_payload.next_payload, self.invalid_ke_payload.next_payload)
        self.assertEqual(parsed_payload.spi, self.invalid_ke_payload.spi)


class TestIkev2PayloadCertificateRequest(unittest.TestCase):
    def setUp(self):
        self.cert_encoding = 4  # X.509 certificate signature
        self.certificate_data = (
            b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
            b'\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
        )

        self.certreq_dict = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x00'),
            ('payload_length', b'\x00\x25'),  # 37 bytes (4 header + 1 cert_encoding + 32 data)
            ('cert_encoding', b'\x04'),  # X.509 certificate signature
            ('certificate_data', self.certificate_data),
        ])
        self.certreq_bytes = b''.join(self.certreq_dict.values())

        self.certreq_payload = Ikev2PayloadCertificateRequest(
            flags=set(),
            cert_encoding=self.cert_encoding,
            certificate_data=self.certificate_data
        )
        self.certreq_payload.next_payload = Ikev2PayloadType.NONE

    def test_get_payload_type(self):
        self.assertEqual(Ikev2PayloadCertificateRequest.get_payload_type(), Ikev2PayloadType.CERTREQ)

    def test_parse(self):
        parsed_certreq: Ikev2PayloadCertificateRequest = Ikev2PayloadCertificateRequest.parse_exact_size(
            self.certreq_bytes)
        self.assertEqual(parsed_certreq.next_payload, Ikev2PayloadType.NONE)
        self.assertEqual(parsed_certreq.flags, set())
        self.assertEqual(parsed_certreq.cert_encoding, self.cert_encoding)
        self.assertEqual(parsed_certreq.certificate_data, self.certificate_data)

    def test_compose(self):
        composed_bytes = self.certreq_payload.compose()
        self.assertEqual(composed_bytes, self.certreq_bytes)

    def test_round_trip(self):
        composed_bytes = self.certreq_payload.compose()
        parsed_payload: Ikev2PayloadCertificateRequest = Ikev2PayloadCertificateRequest.parse_exact_size(
            composed_bytes)

        self.assertEqual(parsed_payload.cert_encoding, self.certreq_payload.cert_encoding)
        self.assertEqual(parsed_payload.certificate_data, self.certreq_payload.certificate_data)
        self.assertEqual(parsed_payload.flags, self.certreq_payload.flags)
        self.assertEqual(parsed_payload.next_payload, self.certreq_payload.next_payload)

    def test_error_parse_not_enough_data(self):
        incomplete_data = self.certreq_bytes[:-5]

        with self.assertRaises(NotEnoughData) as context_manager:
            Ikev2PayloadCertificateRequest.parse_exact_size(incomplete_data)
        self.assertEqual(context_manager.exception.bytes_needed, 5)


class TestIkev2PayloadVendorId(unittest.TestCase):
    def setUp(self):
        self.vendor_id = (
            b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
            b'\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
        )

        self.vendor_dict = collections.OrderedDict([
            ('next_payload', b'\x00'),  # NONE
            ('flags', b'\x80'),  # CRITICAL (0x80)
            ('payload_length', b'\x00\x24'),  # 36 bytes (4 header + 32 vendor_id)
            ('vendor_id', self.vendor_id),
        ])
        self.vendor_bytes = b''.join(self.vendor_dict.values())

        self.vendor_payload = Ikev2PayloadVendorId(
            flags={Ikev2PayloadFlags.CRITICAL},
            vendor_id=self.vendor_id
        )
        self.vendor_payload.next_payload = Ikev2PayloadType.NONE

    def test_get_payload_type(self):
        self.assertEqual(Ikev2PayloadVendorId.get_payload_type(), Ikev2PayloadType.VENDOR_ID)

    def test_parse(self):
        parsed_vendor: Ikev2PayloadVendorId = Ikev2PayloadVendorId.parse_exact_size(self.vendor_bytes)
        self.assertEqual(parsed_vendor.next_payload, Ikev2PayloadType.NONE)
        self.assertEqual(parsed_vendor.flags, {Ikev2PayloadFlags.CRITICAL})
        self.assertEqual(parsed_vendor.vendor_id, self.vendor_id)

    def test_compose(self):
        composed_bytes = self.vendor_payload.compose()
        self.assertEqual(composed_bytes, self.vendor_bytes)

    def test_round_trip(self):
        composed_bytes = self.vendor_payload.compose()
        parsed_payload: Ikev2PayloadVendorId = Ikev2PayloadVendorId.parse_exact_size(composed_bytes)

        self.assertEqual(parsed_payload.vendor_id, self.vendor_payload.vendor_id)
        self.assertEqual(parsed_payload.flags, self.vendor_payload.flags)
        self.assertEqual(parsed_payload.next_payload, self.vendor_payload.next_payload)

    def test_error_parse_not_enough_data(self):
        incomplete_data = self.vendor_bytes[:-10]

        with self.assertRaises(NotEnoughData) as context_manager:
            Ikev2PayloadVendorId.parse_exact_size(incomplete_data)
        self.assertEqual(context_manager.exception.bytes_needed, 10)


class TestTransformAttributeKeyLength(unittest.TestCase):
    def setUp(self):
        self.key_length_value = 128
        self.key_length_attribute = TransformAttributeKeyLength(value=self.key_length_value)

    def test_parse(self):
        composed_bytes = self.key_length_attribute.compose()
        parsed_attribute = TransformAttributeKeyLength.parse_exact_size(composed_bytes)

        self.assertEqual(parsed_attribute.value, self.key_length_attribute.value)

    def test_compose(self):
        composed_bytes = self.key_length_attribute.compose()
        self.assertEqual(len(composed_bytes), 4)
        self.assertEqual(composed_bytes[0], 0x80)
        self.assertEqual(composed_bytes[1], Ikev2TransformAttributeType.KEY_LENGTH.value.code)
        self.assertEqual(composed_bytes[2:4], b'\x00\x80')

    def test_round_trip(self):
        composed_bytes = self.key_length_attribute.compose()
        parsed_attribute = TransformAttributeKeyLength.parse_exact_size(composed_bytes)

        self.assertEqual(parsed_attribute.value, self.key_length_attribute.value)

    def test_error_parse_not_enough_data(self):
        incomplete_data = b'\x80\x0e\x00'

        with self.assertRaises(NotEnoughData) as context_manager:
            TransformAttributeKeyLength.parse_exact_size(incomplete_data)
        self.assertEqual(context_manager.exception.bytes_needed, 1)


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-

import collections
import unittest


from cryptodatahub.common.exception import InvalidValue
from cryptodatahub.common.grade import Grade

from cryptoparser.common.parse import ParserText

from cryptoparser.ssh.version import (
    SshVersion,
    SshProtocolVersion,
    SshSoftwareVersionCryptlib,
    SshSoftwareVersionDropbear,
    SshSoftwareVersionIPSSH,
    SshSoftwareVersionMonacaSSH,
    SshSoftwareVersionOpenSSH,
    SshSoftwareVersionUnparsed,
    SshSoftwareVersionParsedVariant
)


class TestSshVersion(unittest.TestCase):
    def test_error(self):
        parsable = b'3.0'
        expected_error_message = '3 is not a valid SshVersion'
        with self.assertRaisesRegex(ValueError, expected_error_message):
            # pylint: disable=expression-not-assigned
            SshProtocolVersion.parse_exact_size(parsable)

        expected_error_message = '\'.0\' is not a valid SshProtocolVersion'
        with self.assertRaisesRegex(InvalidValue, expected_error_message):
            # pylint: disable=expression-not-assigned
            SshProtocolVersion.parse_exact_size(b'.0')

        expected_error_message = '\'2.\' is not a valid SshProtocolVersion'
        with self.assertRaisesRegex(InvalidValue, expected_error_message):
            # pylint: disable=expression-not-assigned
            SshProtocolVersion.parse_exact_size(b'2.')

    def test_parse(self):
        version = SshProtocolVersion.parse_exact_size(b'1.0')
        self.assertEqual(version, SshProtocolVersion(SshVersion.SSH1))
        self.assertEqual(version.supported_versions, [SshVersion.SSH1, ])

        version = SshProtocolVersion.parse_exact_size(b'1.99')
        self.assertEqual(version, SshProtocolVersion(SshVersion.SSH1, 99))
        self.assertEqual(version.supported_versions, [SshVersion.SSH1, SshVersion.SSH2])

        version = SshProtocolVersion.parse_exact_size(b'2.0')
        self.assertEqual(version, SshProtocolVersion(SshVersion.SSH2))
        self.assertEqual(version.supported_versions, [SshVersion.SSH2, ])

    def test_compose(self):
        self.assertEqual(b'2.0', SshProtocolVersion(SshVersion.SSH2, 0).compose())
        self.assertEqual(b'1.1', SshProtocolVersion(SshVersion.SSH1, 1).compose())

    def test_lt(self):
        self.assertLess(
            SshProtocolVersion(SshVersion.SSH1),
            SshProtocolVersion(SshVersion.SSH2)
        )

        self.assertLess(
            SshProtocolVersion(SshVersion.SSH2, 0),
            SshProtocolVersion(SshVersion.SSH2, 1)
        )

        self.assertLess(
            SshProtocolVersion(SshVersion.SSH1, 1),
            SshProtocolVersion(SshVersion.SSH2, 0)
        )

        self.assertGreater(
            SshProtocolVersion(SshVersion.SSH2, 0),
            SshProtocolVersion(SshVersion.SSH1, 1)
        )

    def test_eq(self):
        self.assertEqual(
            SshProtocolVersion(SshVersion.SSH1, 0),
            SshProtocolVersion(SshVersion.SSH1, 0)
        )

        self.assertEqual(
            SshProtocolVersion(SshVersion.SSH2, 0),
            SshProtocolVersion(SshVersion.SSH2, 0)
        )

    def test_as_json(self):
        self.assertEqual(SshProtocolVersion(SshVersion.SSH1, 0).as_json(), '\"ssh1\"')
        self.assertEqual(SshProtocolVersion(SshVersion.SSH1, 1).as_json(), '\"ssh1\"')
        self.assertEqual(SshProtocolVersion(SshVersion.SSH2, 0).as_json(), '\"ssh2\"')
        self.assertEqual(SshProtocolVersion(SshVersion.SSH2, 1).as_json(), '\"ssh2\"')

    def test_str(self):
        self.assertEqual(str(SshProtocolVersion(SshVersion.SSH1, 0)), 'SSH 1.0')
        self.assertEqual(str(SshProtocolVersion(SshVersion.SSH1, 1)), 'SSH 1.1')
        self.assertEqual(str(SshProtocolVersion(SshVersion.SSH2, 0)), 'SSH 2.0')
        self.assertEqual(str(SshProtocolVersion(SshVersion.SSH2, 1)), 'SSH 2.1')

    def test_grade(self):
        self.assertEqual(SshProtocolVersion(SshVersion.SSH2).grade, Grade.SECURE)
        self.assertEqual(SshProtocolVersion(SshVersion.SSH1).grade, Grade.INSECURE)
        self.assertEqual(SshProtocolVersion(SshVersion.SSH1, 99).grade, Grade.INSECURE)


class TestSshSoftwareVersion(unittest.TestCase):
    @staticmethod
    def _get_software_version(raw):
        parser = ParserText(raw)
        parser.parse_parsable('software_version', SshSoftwareVersionParsedVariant)

        return parser['software_version']

    def test_parse(self):
        software_version = self._get_software_version(b'cryptlib')
        self.assertEqual(software_version.vendor, 'cryptlib')
        self.assertEqual(software_version, SshSoftwareVersionCryptlib())

        software_version = self._get_software_version(b'dropbear_2020.81')
        self.assertEqual(software_version.vendor, 'dropbear')
        self.assertEqual(software_version, SshSoftwareVersionDropbear('2020.81'))

        software_version = self._get_software_version(b'IPSSH-6.9.0')
        self.assertEqual(software_version.vendor, 'IPSSH')
        self.assertEqual(software_version, SshSoftwareVersionIPSSH('6.9.0'))

        software_version = self._get_software_version(b'Monaca')
        self.assertEqual(software_version.vendor, 'Monaca')
        self.assertEqual(software_version, SshSoftwareVersionMonacaSSH())

        software_version = self._get_software_version(b'OpenSSH_8.6')
        self.assertEqual(software_version.vendor, 'OpenSSH')
        self.assertEqual(software_version, SshSoftwareVersionOpenSSH('8.6'))

        parser = ParserText(b'unknown.ssh.server-1.2.3')
        parser.parse_parsable('software_version', SshSoftwareVersionUnparsed)
        software_version = parser['software_version']
        self.assertEqual(software_version.raw, 'unknown.ssh.server-1.2.3')

    def test_compose(self):
        software_version = SshSoftwareVersionCryptlib()
        self.assertEqual(software_version.compose(), b'cryptlib')
        self.assertEqual(
            software_version._asdict(),
            collections.OrderedDict([('vendor', 'cryptlib'), ('version', None)])
        )

        software_version = SshSoftwareVersionDropbear('2020.81')
        self.assertEqual(software_version.compose(), b'dropbear_2020.81')
        self.assertEqual(
            software_version._asdict(),
            collections.OrderedDict([('vendor', 'dropbear'), ('version', '2020.81')])
        )

        software_version = SshSoftwareVersionIPSSH('6.9.0')
        self.assertEqual(software_version.compose(), b'IPSSH-6.9.0')
        self.assertEqual(
            software_version._asdict(),
            collections.OrderedDict([('vendor', 'IPSSH'), ('version', '6.9.0')])
        )

        software_version = SshSoftwareVersionMonacaSSH()
        self.assertEqual(software_version.compose(), b'Monaca')
        self.assertEqual(
            software_version._asdict(),
            collections.OrderedDict([('vendor', 'Monaca'), ('version', None)])
        )

        software_version = SshSoftwareVersionOpenSSH('8.6')
        self.assertEqual(software_version.compose(), b'OpenSSH_8.6')
        self.assertEqual(
            software_version._asdict(),
            collections.OrderedDict([('vendor', 'OpenSSH'), ('version', '8.6')])
        )

        software_version = SshSoftwareVersionUnparsed('unknown.ssh.server-1.2.3')
        self.assertEqual(software_version.compose(), b'unknown.ssh.server-1.2.3')
        self.assertEqual(software_version.as_markdown(), 'unknown.ssh.server-1.2.3')

    def test_error_raw(self):
        with self.assertRaises(InvalidValue):
            SshSoftwareVersionUnparsed('αβγ')
        with self.assertRaises(InvalidValue):
            SshSoftwareVersionUnparsed('software_version ')
        with self.assertRaises(InvalidValue):
            SshSoftwareVersionUnparsed('software_version\r')
        with self.assertRaises(InvalidValue):
            SshSoftwareVersionUnparsed('software_version\n')



# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-

import unittest


from cryptodatahub.common.exception import InvalidValue

from cryptoparser.common.exception import InvalidType, NotEnoughData, TooMuchData


class TestException(unittest.TestCase):
    def test_str(self):
        with self.assertRaisesRegex(
                NotEnoughData, 'not enough data received from target; missing_byte_count="10"'
        ) as context_manager:
            raise NotEnoughData(10)
        self.assertEqual(context_manager.exception.bytes_needed, 10)

        with self.assertRaisesRegex(
                TooMuchData, 'too much data received from target; rest_byte_count="10"'
        ) as context_manager:
            raise TooMuchData(10)
        self.assertEqual(context_manager.exception.bytes_needed, 10)

        with self.assertRaisesRegex(
                InvalidValue, '0xa is not a valid str member name value'
        ) as context_manager:
            raise InvalidValue(10, str, 'member name')
        self.assertEqual(context_manager.exception.value, 10)

        with self.assertRaisesRegex(InvalidValue, '0xa is not a valid str') as context_manager:
            raise InvalidValue(10, str)
        self.assertEqual(context_manager.exception.value, 10)

        with self.assertRaisesRegex(InvalidType, 'invalid type value received from target') as context_manager:
            raise InvalidType(10)



# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-
# pylint: disable=too-many-lines

import collections
import datetime
import ipaddress
import unittest

from collections import OrderedDict

from test.common.classes import TestClasses

from cryptodatahub.common.algorithm import Authentication, Hash, NamedGroup
from cryptodatahub.common.key import (
    PublicKey,
    PublicKeySize,
    PublicKeyParamsDsa,
    PublicKeyParamsEcdsa,
    PublicKeyParamsEddsa,
    PublicKeyParamsRsa,
)
from cryptodatahub.common.exception import InvalidValue

from cryptodatahub.ssh.algorithm import SshHostKeyAlgorithm

from cryptoparser.common.exception import InvalidType, NotEnoughData

from cryptoparser.ssh.key import (
    SshCertType,
    SshCertExtensionVector,
    SshCertExtensionForceCommand,
    SshCertExtensionNoPrecenseRequired,
    SshCertExtensionPermitX11Forwarding,
    SshCertExtensionPermitAgentForwarding,
    SshCertExtensionPermitPortForwarding,
    SshCertExtensionPermitPTY,
    SshCertExtensionPermitUserRC,
    SshCertExtensionSourceAddress,
    SshCertExtensionUnparsed,
    SshCertConstraintVector,
    SshCertCriticalOptionVector,
    SshCertSignature,
    SshCertValidPrincipals,
    SshHostPublicKeyVariant,
    SshHostCertificateV00DSS,
    SshHostCertificateV00RSA,
    SshHostCertificateV01DSS,
    SshHostCertificateV01ECDSA,
    SshHostCertificateV01EDDSA,
    SshHostCertificateV01RSA,
    SshHostKeyDSS,
    SshHostKeyECDSA,
    SshHostKeyEDDSA,
    SshHostKeyRSA,
    SshString,
    SshX509Certificate,
    SshX509CertificateChain,
)


class TestPublicKeyBase(unittest.TestCase):
    def test_error(self):
        with self.assertRaises(NotEnoughData) as context_manager:
            SshHostKeyRSA.parse_exact_size(b'')
        self.assertEqual(context_manager.exception.bytes_needed, 4)

        with self.assertRaises(InvalidValue) as context_manager:
            SshHostKeyRSA.parse_exact_size(b'\x00\x00\x00\x16' + b'non-existing-type-name')
        self.assertEqual(context_manager.exception.value, 'non-existing-type-name')


class TestString(unittest.TestCase):
    def setUp(self):
        self.string_bytes = bytes(
            b'\x00\x00\x00\x06' +
            b'string' +
            b''
        )
        self.string = SshString('string')

    def test_parse(self):
        string = SshString.parse_exact_size(self.string_bytes)
        self.assertEqual(string.value, 'string')

    def test_compose(self):
        self.assertEqual(self.string.compose(), self.string_bytes)


class TestHostKeyDSS(TestPublicKeyBase):
    def setUp(self):
        self.host_key_bytes = bytes(
            b'\x00\x00\x00\x07' +               # host_key_algorithm_length
            b'ssh-dss' +                        # host_key_algorithm
            b'\x00\x00\x00\x04' +               # p_length
            b'\x01\x01\x02\x03' +               # p
            b'\x00\x00\x00\x04' +               # q_length
            b'\x04\x05\x06\x07' +               # q
            b'\x00\x00\x00\x04' +               # g_length
            b'\x08\x09\x0a\x0b' +               # g
            b'\x00\x00\x00\x04' +               # y_length
            b'\x0c\x0d\x0e\x0f' +               # y
            b''
        )
        self.host_key = SshHostKeyDSS(
            host_key_algorithm=SshHostKeyAlgorithm.SSH_DSS,
            public_key=PublicKey.from_params(PublicKeyParamsDsa(
                prime=0x01010203,
                order=0x04050607,
                generator=0x08090a0b,
                public_key_value=0x0c0d0e0f,
            )),
        )

    def test_parse(self):
        host_key = SshHostPublicKeyVariant.parse_exact_size(self.host_key_bytes)
        self.assertEqual(host_key.public_key.params.prime, 0x01010203)
        self.assertEqual(host_key.public_key.params.order, 0x04050607)
        self.assertEqual(host_key.public_key.params.generator, 0x08090a0b)
        self.assertEqual(host_key.public_key.params.public_key_value, 0x0c0d0e0f)
        self.assertEqual(host_key.public_key.key_size, 32)

    def test_compose(self):
        self.assertEqual(self.host_key.compose(), self.host_key_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_key._asdict(), OrderedDict([
            ('key_type', 'host key'),
            ('algorithm', Authentication.DSS),
            ('size', PublicKeySize(Authentication.DSS, 32)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:wdClb94C9Lyi38P1o/SEG38glOh3ea5CJl84bZVx2yM='),
                (Hash.SHA1, 'SHA1:fOmDMlRkSkplVc2vGTmkRY65j/c='),
                (Hash.MD5, 'MD5:f2:4f:70:62:fc:36:fa:20:25:62:5d:95:1c:6c:5e:63'),
            ])),
            ('known_hosts', 'AAAAB3NzaC1kc3MAAAAEAQECAwAAAAQEBQYHAAAABAgJCgsAAAAEDA0ODw=='),
            ('host_key_algorithm', SshHostKeyAlgorithm.SSH_DSS),
            ('public_key', PublicKey.from_params(PublicKeyParamsDsa(
                prime=0x01010203,
                generator=0x08090a0b,
                order=0x04050607,
                public_key_value=0x0c0d0e0f,
            ))),
        ]))


class TestHostKeyRSA(TestPublicKeyBase):
    def setUp(self):
        self.host_key_bytes = bytes(
            b'\x00\x00\x00\x07' +               # host_key_algorithm_length
            b'ssh-rsa' +                        # host_key_algorithm
            b'\x00\x00\x00\x04' +               # e_length
            b'\x01\x01\x02\x03' +               # e
            b'\x00\x00\x00\x04' +               # n_length
            b'\x04\x05\x06\x07' +               # n
            b''
        )
        self.host_key = SshHostKeyRSA(
            host_key_algorithm=SshHostKeyAlgorithm.SSH_RSA,
            public_key=PublicKey.from_params(PublicKeyParamsRsa(
                modulus=0x04050607,
                public_exponent=0x01010203,
            )),
        )

    def test_parse(self):
        host_key = SshHostPublicKeyVariant.parse_exact_size(self.host_key_bytes)
        public_key_params = host_key.public_key.params
        self.assertEqual(public_key_params.public_exponent, 0x01010203)
        self.assertEqual(public_key_params.modulus, 0x04050607)
        self.assertEqual(host_key.public_key.key_size, 32)

    def test_compose(self):
        self.assertEqual(self.host_key.compose(), self.host_key_bytes)

    def test_key_size(self):
        self.assertEqual(self.host_key.key_size, PublicKeySize(Authentication.RSA, 32))

    def test_asdict(self):
        self.assertEqual(self.host_key._asdict(), OrderedDict([
            ('key_type', 'host key'),
            ('algorithm', Authentication.RSA),
            ('size', PublicKeySize(Authentication.RSA, 32)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:ZuSq5GtQTjPj8LAwY4UE4gGILhIAh5kDaDkkEYLaRU0='),
                (Hash.SHA1, 'SHA1:KAG3KmsLUs4OClEUj62npdXcJTg='),
                (Hash.MD5, 'MD5:0b:40:11:ce:71:86:01:02:2c:7c:9e:13:d9:37:3b:aa'),
            ])),
            ('known_hosts', 'AAAAB3NzaC1yc2EAAAAEAQECAwAAAAQEBQYH'),
        ]))


class TestHostKeyECDSA(TestPublicKeyBase):
    def setUp(self):
        self.point_x_bytes = b'\x80' + (256 // 8 - 1) * b'\x00'
        self.point_y_bytes = b'\x40' + (256 // 8 - 1) * b'\x00'
        self.host_key_bytes = bytes(
            b'\x00\x00\x00\x13' +                  # host_key_algorithm_length
            b'ecdsa-sha2-nistp256' +               # host_key_algorithm
            b'\x00\x00\x00\x08' +                  # curve_name_length
            b'nistp256' +                          # curve_name
            b'\x00\x00\x00\x41' +                  # curve_data_length
            b'\04' +                               # curve_data
            self.point_x_bytes +
            self.point_y_bytes +
            b''
        )
        self.host_key = SshHostKeyECDSA(
            host_key_algorithm=SshHostKeyAlgorithm.ECDSA_SHA2_NISTP256,
            public_key=PublicKey.from_params(PublicKeyParamsEcdsa(
                named_group=NamedGroup.PRIME256V1,
                point_x=2 ** 255,
                point_y=2 ** 254,
            )),
        )

    def test_parse(self):
        host_key = SshHostPublicKeyVariant.parse_exact_size(self.host_key_bytes)
        self.assertEqual(host_key.public_key.params.named_group, NamedGroup.PRIME256V1)
        self.assertEqual(host_key.public_key.params.point_x, 2 ** 255)
        self.assertEqual(host_key.public_key.params.point_y, 2 ** 254)
        self.assertEqual(host_key.public_key.key_size, 256)

    def test_compose(self):
        self.assertEqual(self.host_key.compose(), self.host_key_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_key._asdict(), OrderedDict([
            ('key_type', 'host key'),
            ('algorithm', Authentication.ECDSA),
            ('size', PublicKeySize(Authentication.ECDSA, 256)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:+baTTAvJKIn0rfi1HVlDxDb/lIzi41H9UoCkFPyyO4I='),
                (Hash.SHA1, 'SHA1:WiBKhHvCyV8LpdXgJWrJr9WAhqw='),
                (Hash.MD5, 'MD5:86:c6:d5:ca:3e:5e:82:95:31:80:8a:30:b3:a3:6e:80'),
            ])),
            ('known_hosts', (
                'AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBIAAAAAAAAAA'
                'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAQAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                'AAAAAAAAAAA='
            )),
        ]))


class TestHostKeyEDDSA(TestPublicKeyBase):
    def setUp(self):
        self.host_key_bytes = bytes(
            b'\x00\x00\x00\x0b' +               # host_key_algorithm_length
            b'ssh-ed25519' +                    # host_key_algorithm
            b'\x00\x00\x00\x20' +               # key_data_length
            b'\x00\x01\x02\x03' * 8 +           # key_data
            b''
        )
        self.host_key = SshHostKeyEDDSA(
            host_key_algorithm=SshHostKeyAlgorithm.SSH_ED25519,
            public_key=PublicKey.from_params(PublicKeyParamsEddsa(
                curve_type=NamedGroup.CURVE25519,
                key_data=b'\x00\x01\x02\x03' * 8,
            )),
        )

    def test_parse(self):
        host_key = SshHostPublicKeyVariant.parse_exact_size(self.host_key_bytes)
        self.assertEqual(host_key.public_key.params.key_data, b'\x00\x01\x02\x03' * 8)
        self.assertEqual(host_key.public_key.key_size, 256)

    def test_compose(self):
        self.assertEqual(self.host_key.compose(), self.host_key_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_key._asdict(), OrderedDict([
            ('key_type', 'host key'),
            ('algorithm', Authentication.EDDSA),
            ('size', PublicKeySize(Authentication.EDDSA, 256)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:tE7ReEqO7s6dpo8PFRQXhAe4Vdy9HkT0UawUx+NfClk='),
                (Hash.SHA1, 'SHA1:0ql0OFSSY06SHsZhtEAJ1Wx2AUo='),
                (Hash.MD5, 'MD5:5c:83:5d:46:c1:7e:b0:47:70:6c:2c:30:a1:28:87:c0'),
            ])),
            ('known_hosts', 'AAAAC3NzaC1lZDI1NTE5AAAAIAABAgMAAQIDAAECAwABAgMAAQIDAAECAwABAgMAAQID'),
        ]))


class TestCertType(unittest.TestCase):
    def test_as_markdown(self):
        self.assertEqual(SshCertType.SSH_CERT_TYPE_USER.value.as_markdown(), 'User')
        self.assertEqual(SshCertType.SSH_CERT_TYPE_HOST.value.as_markdown(), 'Host')


class TestCertExtensionUnparsed(unittest.TestCase):
    def setUp(self):
        self.extension_dict = collections.OrderedDict([
            ('extension_name_length', b'\x00\x00\x00\x10'),
            ('extension_name', b'extension name 1'),
            ('extension_data_length', b'\x00\x00\x00\x10'),
            ('extension_data', b'extension data 1'),
        ])
        self.extension_bytes = b''.join(self.extension_dict.values())

        self.extension = SshCertExtensionUnparsed(
            'extension name 1',
            b'extension data 1',
        )

    def test_parse(self):
        extension = SshCertExtensionUnparsed.parse_exact_size(self.extension_bytes)
        self.assertEqual(extension.extension_name, self.extension.extension_name)
        self.assertEqual(extension.extension_data, self.extension.extension_data)

    def test_compose(self):
        self.assertEqual(self.extension.compose(), self.extension_bytes)


class TestHostCertExtensionsUnparsed(unittest.TestCase):
    def setUp(self):
        self.extensions_bytes = (
            b'\x00\x00\x00\x50' +
            b'\x00\x00\x00\x10' +
            b'extension name 1' +
            b'\x00\x00\x00\x10' +
            b'extension data 1' +
            b'\x00\x00\x00\x10' +
            b'extension name 2' +
            b'\x00\x00\x00\x10' +
            b'extension data 2' +
            b''
        )
        self.extensions = SshCertExtensionVector([
            SshCertExtensionUnparsed('extension name 1', b'extension data 1'),
            SshCertExtensionUnparsed('extension name 2', b'extension data 2'),
        ])

    def test_parse(self):
        self.assertEqual(
            SshCertExtensionVector.parse_exact_size(self.extensions_bytes),
            self.extensions
        )

    def test_compose(self):
        self.assertEqual(self.extensions.compose(), self.extensions_bytes)


class TestHostCertExtensionsNoData(unittest.TestCase):
    def setUp(self):
        self.extensions_bytes = (
            b'\x00\x00\x00\x9e' +
            b'\x00\x00\x00\x14' +
            b'no-presence-required' +
            b'\x00\x00\x00\x00' +
            b'\x00\x00\x00\x15' +
            b'permit-X11-forwarding' +
            b'\x00\x00\x00\x00' +
            b'\x00\x00\x00\x17' +
            b'permit-agent-forwarding' +
            b'\x00\x00\x00\x00' +
            b'\x00\x00\x00\x16' +
            b'permit-port-forwarding' +
            b'\x00\x00\x00\x00' +
            b'\x00\x00\x00\x0a' +
            b'permit-pty' +
            b'\x00\x00\x00\x00' +
            b'\x00\x00\x00\x0e' +
            b'permit-user-rc' +
            b'\x00\x00\x00\x00' +
            b''
        )
        self.extensions = SshCertExtensionVector([
            SshCertExtensionNoPrecenseRequired(),
            SshCertExtensionPermitX11Forwarding(),
            SshCertExtensionPermitAgentForwarding(),
            SshCertExtensionPermitPortForwarding(),
            SshCertExtensionPermitPTY(),
            SshCertExtensionPermitUserRC(),
        ])

    def test_parse(self):
        self.assertEqual(
            SshCertExtensionVector.parse_exact_size(self.extensions_bytes),
            self.extensions
        )

    def test_compose(self):
        self.assertEqual(self.extensions.compose(), self.extensions_bytes)


class TestHostCertExtensionsWithData(unittest.TestCase):
    def setUp(self):
        self.extensions_bytes = (
            b'\x00\x00\x00\x4b' +
            b'\x00\x00\x00\x0d' +
            b'force-command'
            b'\x00\x00\x00\x07' +
            b'command'
            b'\x00\x00\x00\x0e' +
            b'source-address' +
            b'\x00\x00\x00\x19' +
            b'192.168.0.0/16,10.0.0.0/8' +
            b''
        )
        self.extensions = SshCertCriticalOptionVector([
            SshCertExtensionForceCommand('command'),
            SshCertExtensionSourceAddress([
                ipaddress.IPv4Network('192.168.0.0/16'),
                ipaddress.IPv4Network('10.0.0.0/8'),
            ]),
        ])

    def test_parse(self):
        self.assertEqual(
            SshCertCriticalOptionVector.parse_exact_size(self.extensions_bytes),
            self.extensions
        )

    def test_compose(self):
        self.assertEqual(self.extensions.compose(), self.extensions_bytes)


class TestHostCertificateDSSBase(TestPublicKeyBase):
    def setUp(self):
        self.host_key_bytes = bytes(
            b'\x00\x00\x00\x07' +  # certificate_type
            b'ssh-dss' +
            b'\x00\x00\x00\x04' +  # p_length
            b'\x01\x01\x02\x03' +  # p
            b'\x00\x00\x00\x04' +  # q_length
            b'\x04\x05\x06\x07' +  # q
            b'\x00\x00\x00\x04' +  # g_length
            b'\x08\x09\x0a\x0b' +  # g
            b'\x00\x00\x00\x04' +  # y_length
            b'\x0c\x0d\x0e\x0f' +  # y
            b'\x00\x00\x00\x13' +
            b'\x00\x00\x00\x07' +  # signature_type
            b'ssh-dss' +
            b'\x00\x00\x00\x04' +  # signature_data
            b'\x00\x01\x02\x03' +
            b''
        )


class TestHostCertificateV00DSS(TestHostCertificateDSSBase):
    def setUp(self):
        super().setUp()
        self.host_cert_bytes = bytes(
            b'\x00\x00\x00\x1c' +
            b'ssh-dss-cert-v00@openssh.com' +
            b'\x00\x00\x00\x04' +  # p_length
            b'\x01\x01\x02\x03' +  # p
            b'\x00\x00\x00\x04' +  # q_length
            b'\x04\x05\x06\x07' +  # q
            b'\x00\x00\x00\x04' +  # g_length
            b'\x08\x09\x0a\x0b' +  # g
            b'\x00\x00\x00\x04' +  # y_length
            b'\x0c\x0d\x0e\x0f' +  # y
            b'\x00\x00\x00\x02' +  # certificate_type (SshCertType.SSH_CERT_TYPE_HOST)
            b'\x00\x00\x00\x08' +  # key_id
            b'\x00\x01\x02\x03\x04\x05\x06\x07' +
            b'\x00\x00\x00\x00' +  # valid_principals
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +  # valid_after
            b'\xff\xff\xff\xff\xff\xff\xff\xff' +  # valid_before
            b'\x00\x00\x00\x00' +  # constraints
            b'\x00\x00\x00\x04' +  # nonce
            b'\x00\x01\x02\x03' +
            b'\x00\x00\x00\x00' +  # reserved
            b'\x00\x00\x00\x2b' +  # signature_key
            self.host_key_bytes +
            b''
        )
        self.host_cert = SshHostCertificateV00DSS(
            host_key_algorithm=SshHostKeyAlgorithm.SSH_DSS_CERT_V00_OPENSSH_COM,
            nonce=b'\x00\x01\x02\x03',
            public_key=PublicKey.from_params(PublicKeyParamsDsa(
                prime=0x01010203,
                generator=0x08090a0b,
                order=0x04050607,
                public_key_value=0x0c0d0e0f,
            )),
            certificate_type=SshCertType.SSH_CERT_TYPE_HOST,
            key_id='\x00\x01\x02\x03\x04\x05\x06\x07',
            valid_principals=SshCertValidPrincipals([]),
            valid_after=datetime.datetime.fromtimestamp(0, datetime.timezone.utc),
            valid_before=None,
            constraints=SshCertConstraintVector([]),
            reserved=b'',
            signature_key=SshHostKeyDSS(
                SshHostKeyAlgorithm.SSH_DSS,
                PublicKey.from_params(PublicKeyParamsDsa(
                    prime=0x01010203,
                    generator=0x08090a0b,
                    order=0x04050607,
                    public_key_value=0x0c0d0e0f,
                ))
            ),
            signature=SshCertSignature(
                SshHostKeyAlgorithm.SSH_DSS,
                b'\x00\x01\x02\x03',
            )
        )

    def test_parse(self):
        host_cert = SshHostPublicKeyVariant.parse_exact_size(self.host_cert_bytes)

        self.assertEqual(host_cert.host_key_algorithm, self.host_cert.host_key_algorithm)
        self.assertEqual(host_cert.nonce, self.host_cert.nonce)
        self.assertEqual(
            host_cert.public_key.params.prime, self.host_cert.public_key.params.prime
        )
        self.assertEqual(
            host_cert.public_key.params.order, self.host_cert.public_key.params.order
        )
        self.assertEqual(
            host_cert.public_key.params.generator, self.host_cert.public_key.params.generator
        )
        self.assertEqual(
            host_cert.public_key.params.public_key_value, self.host_cert.public_key.params.public_key_value
        )
        self.assertEqual(host_cert.certificate_type, self.host_cert.certificate_type)
        self.assertEqual(host_cert.key_id, self.host_cert.key_id)
        self.assertEqual(host_cert.valid_principals, self.host_cert.valid_principals)
        self.assertEqual(host_cert.valid_after, self.host_cert.valid_after)
        self.assertEqual(host_cert.valid_before, self.host_cert.valid_before)
        self.assertEqual(host_cert.constraints, self.host_cert.constraints)
        self.assertEqual(host_cert.reserved, self.host_cert.reserved)
        self.assertEqual(host_cert.signature_key, self.host_cert.signature_key)
        self.assertEqual(host_cert.signature, self.host_cert.signature)

    def test_compose(self):
        self.assertEqual(self.host_cert.compose(), self.host_cert_bytes)
        self.assertEqual(self.host_cert.key_bytes, self.host_cert_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_cert._asdict(), OrderedDict([
            ('key_type', 'host certificate'),
            ('algorithm', Authentication.DSS),
            ('size', PublicKeySize(Authentication.DSS, 32)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:JLbl6U9Dd3zrR/pS86OLLJUsL6NU7NOnSd554vL1md0='),
                (Hash.SHA1, 'SHA1:eGd4yAwFTXy+Wdi5xsuEJUNsj0M='),
                (Hash.MD5, 'MD5:01:63:07:cb:45:12:bf:96:4c:fd:62:cd:40:6d:e0:99')
            ])),
            ('known_hosts', (
                'AAAAHHNzaC1kc3MtY2VydC12MDBAb3BlbnNzaC5jb20AAAAEAQECAwAAAAQEBQYH'
                'AAAABAgJCgsAAAAEDA0ODwAAAAIAAAAIAAECAwQFBgcAAAAAAAAAAAAAAAD/////'
                '/////wAAAAAAAAAEAAECAwAAAAAAAAArAAAAB3NzaC1kc3MAAAAEAQECAwAAAAQE'
                'BQYHAAAABAgJCgsAAAAEDA0ODwAAABMAAAAHc3NoLWRzcwAAAAQAAQID'
            )),
            ('host_key_algorithm', SshHostKeyAlgorithm.SSH_DSS_CERT_V00_OPENSSH_COM),
            ('public_key', PublicKey.from_params(PublicKeyParamsDsa(
                prime=0x01010203,
                generator=0x08090a0b,
                order=0x04050607,
                public_key_value=0x0c0d0e0f,
            ))),
            ('certificate_type', SshCertType.SSH_CERT_TYPE_HOST),
            ('key_id', '\x00\x01\x02\x03\x04\x05\x06\x07'),
            ('valid_principals', SshCertValidPrincipals([])),
            ('valid_after', datetime.datetime.fromtimestamp(0, datetime.timezone.utc)),
            ('valid_before', None),
            ('constraints', SshCertConstraintVector([])),
            ('nonce', b'\x00\x01\x02\x03'),
            ('reserved', b''),
            ('signature_key', SshHostKeyDSS(
                host_key_algorithm=SshHostKeyAlgorithm.SSH_DSS,
                public_key=PublicKey.from_params(PublicKeyParamsDsa(
                    prime=0x01010203,
                    generator=0x08090a0b,
                    order=0x04050607,
                    public_key_value=0x0c0d0e0f,
                )),
            )),
            ('signature', SshCertSignature(
                signature_type=SshHostKeyAlgorithm.SSH_DSS,
                signature_data=b'\x00\x01\x02\x03'
            ))
        ]))


class TestHostCertificateV01DSS(TestHostCertificateDSSBase):
    def setUp(self):
        super().setUp()
        self.host_cert_bytes = bytes(
            b'\x00\x00\x00\x1c' +
            b'ssh-dss-cert-v01@openssh.com' +
            b'\x00\x00\x00\x04' +  # nonce
            b'\x00\x01\x02\x03' +
            b'\x00\x00\x00\x04' +  # p_length
            b'\x01\x01\x02\x03' +  # p
            b'\x00\x00\x00\x04' +  # q_length
            b'\x04\x05\x06\x07' +  # q
            b'\x00\x00\x00\x04' +  # g_length
            b'\x08\x09\x0a\x0b' +  # g
            b'\x00\x00\x00\x04' +  # y_length
            b'\x0c\x0d\x0e\x0f' +  # y
            b'\x01\x02\x03\x04\x05\x06\x07\x08' +  # serial
            b'\x00\x00\x00\x02' +  # certificate_type (SshCertType.SSH_CERT_TYPE_HOST)
            b'\x00\x00\x00\x08' +  # key_id
            b'\x00\x01\x02\x03\x04\x05\x06\x07' +
            b'\x00\x00\x00\x00' +  # valid_principals
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +  # valid_after
            b'\xff\xff\xff\xff\xff\xff\xff\xff' +  # valid_before
            b'\x00\x00\x00\x00' +  # critical_options
            b'\x00\x00\x00\x00' +  # extensions
            b'\x00\x00\x00\x00' +  # reserved
            b'\x00\x00\x00\x2b' +  # signature_key
            self.host_key_bytes +
            b''
        )
        self.host_cert = SshHostCertificateV01DSS(
            host_key_algorithm=SshHostKeyAlgorithm.SSH_DSS_CERT_V01_OPENSSH_COM,
            nonce=b'\x00\x01\x02\x03',
            public_key=PublicKey.from_params(PublicKeyParamsDsa(
                prime=0x01010203,
                generator=0x08090a0b,
                order=0x04050607,
                public_key_value=0x0c0d0e0f,
            )),
            serial=0x0102030405060708,
            certificate_type=SshCertType.SSH_CERT_TYPE_HOST,
            key_id='\x00\x01\x02\x03\x04\x05\x06\x07',
            valid_principals=SshCertValidPrincipals([]),
            valid_after=datetime.datetime.fromtimestamp(0, datetime.timezone.utc),
            valid_before=None,
            critical_options=SshCertCriticalOptionVector([]),
            extensions=SshCertExtensionVector([]),
            reserved=b'',
            signature_key=SshHostKeyDSS(
                SshHostKeyAlgorithm.SSH_DSS,
                PublicKey.from_params(PublicKeyParamsDsa(
                    prime=0x01010203,
                    generator=0x08090a0b,
                    order=0x04050607,
                    public_key_value=0x0c0d0e0f,
                )),
            ),
            signature=SshCertSignature(
                SshHostKeyAlgorithm.SSH_DSS,
                b'\x00\x01\x02\x03',
            )
        )

    def test_parse(self):
        host_cert = SshHostPublicKeyVariant.parse_exact_size(self.host_cert_bytes)

        self.assertEqual(host_cert.host_key_algorithm, self.host_cert.host_key_algorithm)
        self.assertEqual(host_cert.nonce, self.host_cert.nonce)
        self.assertEqual(
            host_cert.public_key.params.prime, self.host_cert.public_key.params.prime
        )
        self.assertEqual(
            host_cert.public_key.params.order, self.host_cert.public_key.params.order
        )
        self.assertEqual(
            host_cert.public_key.params.generator, self.host_cert.public_key.params.generator
        )
        self.assertEqual(
            host_cert.public_key.params.public_key_value, self.host_cert.public_key.params.public_key_value
        )
        self.assertEqual(host_cert.certificate_type, self.host_cert.certificate_type)
        self.assertEqual(host_cert.key_id, self.host_cert.key_id)
        self.assertEqual(host_cert.valid_principals, self.host_cert.valid_principals)
        self.assertEqual(host_cert.valid_after, self.host_cert.valid_after)
        self.assertEqual(host_cert.valid_before, self.host_cert.valid_before)
        self.assertEqual(host_cert.critical_options, self.host_cert.critical_options)
        self.assertEqual(host_cert.extensions, SshCertExtensionVector([]))
        self.assertEqual(host_cert.reserved, self.host_cert.reserved)
        self.assertEqual(host_cert.signature_key, self.host_cert.signature_key)
        self.assertEqual(host_cert.signature, self.host_cert.signature)

    def test_compose(self):
        self.assertEqual(self.host_cert.compose(), self.host_cert_bytes)
        self.assertEqual(self.host_cert.key_bytes, self.host_cert_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_cert._asdict(), OrderedDict([
            ('key_type', 'host certificate'),
            ('algorithm', Authentication.DSS),
            ('size', PublicKeySize(Authentication.DSS, 32)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:81ny5W3AQivvf/ffzLM0b13/eAG82/GsSFNaBnVOF4A='),
                (Hash.SHA1, 'SHA1:Kx6Nzq+MWzKiT4q0lxdQbZzecSo='),
                (Hash.MD5, 'MD5:94:9b:e9:f1:27:4c:7a:60:ba:5b:da:47:99:46:13:06')
            ])),
            ('known_hosts', (
                'AAAAHHNzaC1kc3MtY2VydC12MDFAb3BlbnNzaC5jb20AAAAEAAECAwAAAAQBAQID'
                'AAAABAQFBgcAAAAECAkKCwAAAAQMDQ4PAQIDBAUGBwgAAAACAAAACAABAgMEBQYH'
                'AAAAAAAAAAAAAAAA//////////8AAAAAAAAAAAAAAAAAAAArAAAAB3NzaC1kc3MA'
                'AAAEAQECAwAAAAQEBQYHAAAABAgJCgsAAAAEDA0ODwAAABMAAAAHc3NoLWRzcwAA'
                'AAQAAQID'
            )),
            ('host_key_algorithm', SshHostKeyAlgorithm.SSH_DSS_CERT_V01_OPENSSH_COM),
            ('public_key', PublicKey.from_params(PublicKeyParamsDsa(
                prime=0x01010203,
                generator=0x08090a0b,
                order=0x04050607,
                public_key_value=0x0c0d0e0f,
            ))),
            ('nonce', b'\x00\x01\x02\x03'),
            ('serial', 0x0102030405060708),
            ('certificate_type', SshCertType.SSH_CERT_TYPE_HOST),
            ('key_id', '\x00\x01\x02\x03\x04\x05\x06\x07'),
            ('valid_principals', SshCertValidPrincipals([])),
            ('valid_after', datetime.datetime.fromtimestamp(0, datetime.timezone.utc)),
            ('valid_before', None),
            ('critical_options', SshCertCriticalOptionVector([])),
            ('extensions', SshCertExtensionVector([])),
            ('reserved', b''),
            ('signature_key', SshHostKeyDSS(
                host_key_algorithm=SshHostKeyAlgorithm.SSH_DSS,
                public_key=PublicKey.from_params(PublicKeyParamsDsa(
                    prime=0x01010203,
                    generator=0x08090a0b,
                    order=0x04050607,
                    public_key_value=0x0c0d0e0f,
                )),
            )),
            ('signature', SshCertSignature(
                signature_type=SshHostKeyAlgorithm.SSH_DSS,
                signature_data=b'\x00\x01\x02\x03'
            ))
        ]))


class TestHostCertificateRSABase(TestPublicKeyBase):
    def setUp(self):
        self.host_key_bytes = bytes(
            b'\x00\x00\x00\x07' +  # certificate_type
            b'ssh-rsa' +
            b'\x00\x00\x00\x03' +  # e
            b'\x01\x00\x01' +
            b'\x00\x00\x00\x04' +  # n
            b'\x01\x01\x02\x03' +
            b'\x00\x00\x00\x13' +
            b'\x00\x00\x00\x07' +  # signature_type
            b'ssh-rsa' +
            b'\x00\x00\x00\x04' +  # signature_data
            b'\x00\x01\x02\x03' +
            b''
        )


class TestHostCertificateV00RSA(TestHostCertificateRSABase):
    def setUp(self):
        super().setUp()
        self.host_cert_bytes = bytes(
            b'\x00\x00\x00\x1c' +
            b'ssh-rsa-cert-v00@openssh.com' +
            b'\x00\x00\x00\x01' +  # e
            b'\x03' +
            b'\x00\x00\x00\x04' +  # n
            b'\x01\x01\x02\x03' +
            b'\x00\x00\x00\x02' +  # certificate_type (SshCertType.SSH_CERT_TYPE_HOST)
            b'\x00\x00\x00\x08' +  # key_id
            b'\x00\x01\x02\x03\x04\x05\x06\x07' +
            b'\x00\x00\x00\x00' +  # valid_principals
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +  # valid_after
            b'\xff\xff\xff\xff\xff\xff\xff\xff' +  # valid_before
            b'\x00\x00\x00\x00' +  # constraints
            b'\x00\x00\x00\x04' +  # nonce
            b'\x00\x01\x02\x03' +
            b'\x00\x00\x00\x00' +  # reserved
            b'\x00\x00\x00\x1a' +  # signature_key
            self.host_key_bytes +
            b''
        )
        self.host_cert = SshHostCertificateV00RSA(
            host_key_algorithm=SshHostKeyAlgorithm.SSH_RSA_CERT_V00_OPENSSH_COM,
            nonce=b'\x00\x01\x02\x03',
            public_key=PublicKey.from_params(PublicKeyParamsRsa(
                public_exponent=0x03,
                modulus=0x01010203,
            )),
            certificate_type=SshCertType.SSH_CERT_TYPE_HOST,
            key_id='\x00\x01\x02\x03\x04\x05\x06\x07',
            valid_principals=SshCertValidPrincipals([]),
            valid_after=datetime.datetime.fromtimestamp(0, datetime.timezone.utc),
            valid_before=None,
            constraints=SshCertConstraintVector([]),
            reserved=b'',
            signature_key=SshHostKeyRSA(
                SshHostKeyAlgorithm.SSH_RSA,
                PublicKey.from_params(PublicKeyParamsRsa(
                    public_exponent=0x010001,
                    modulus=0x01010203,
                )),
            ),
            signature=SshCertSignature(
                SshHostKeyAlgorithm.SSH_RSA,
                b'\x00\x01\x02\x03',
            )
        )

    def test_parse(self):
        host_cert = SshHostPublicKeyVariant.parse_exact_size(self.host_cert_bytes)

        self.assertEqual(host_cert.host_key_algorithm, self.host_cert.host_key_algorithm)
        self.assertEqual(host_cert.nonce, self.host_cert.nonce)
        self.assertEqual(host_cert.public_key.params.public_exponent, self.host_cert.public_key.params.public_exponent)
        self.assertEqual(host_cert.public_key.params.modulus, self.host_cert.public_key.params.modulus)
        self.assertEqual(host_cert.certificate_type, self.host_cert.certificate_type)
        self.assertEqual(host_cert.key_id, self.host_cert.key_id)
        self.assertEqual(host_cert.valid_principals, self.host_cert.valid_principals)
        self.assertEqual(host_cert.valid_after, self.host_cert.valid_after)
        self.assertEqual(host_cert.valid_before, self.host_cert.valid_before)
        self.assertEqual(host_cert.constraints, self.host_cert.constraints)
        self.assertEqual(host_cert.reserved, self.host_cert.reserved)
        self.assertEqual(host_cert.signature_key, self.host_cert.signature_key)
        self.assertEqual(host_cert.signature, self.host_cert.signature)

    def test_compose(self):
        self.assertEqual(self.host_cert.compose(), self.host_cert_bytes)
        self.assertEqual(self.host_cert.key_bytes, self.host_cert_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_cert._asdict(), OrderedDict([
            ('key_type', 'host certificate'),
            ('algorithm', Authentication.RSA),
            ('size', PublicKeySize(Authentication.RSA, 32)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:YJVpL0zTCstfPryV5C1tD3boAzBrzRlAMjrAosxw4pA='),
                (Hash.SHA1, 'SHA1:zNRNUIcyRZvu8MuhAmtALdUNCMM='),
                (Hash.MD5, 'MD5:9c:ce:d6:f1:7f:96:d1:5f:9d:14:a3:32:74:a1:18:88')
            ])),
            ('known_hosts', (
                'AAAAHHNzaC1yc2EtY2VydC12MDBAb3BlbnNzaC5jb20AAAABAwAAAAQBAQIDAAAA'
                'AgAAAAgAAQIDBAUGBwAAAAAAAAAAAAAAAP//////////AAAAAAAAAAQAAQIDAAAA'
                'AAAAABoAAAAHc3NoLXJzYQAAAAMBAAEAAAAEAQECAwAAABMAAAAHc3NoLXJzYQAA'
                'AAQAAQID'
            )),
            ('host_key_algorithm', SshHostKeyAlgorithm.SSH_RSA_CERT_V00_OPENSSH_COM),
            ('public_key', PublicKey.from_params(PublicKeyParamsRsa(
                public_exponent=0x03,
                modulus=0x01010203,
            ))),
            ('certificate_type', SshCertType.SSH_CERT_TYPE_HOST),
            ('key_id', '\x00\x01\x02\x03\x04\x05\x06\x07'),
            ('valid_principals', SshCertValidPrincipals([])),
            ('valid_after', datetime.datetime.fromtimestamp(0, datetime.timezone.utc)),
            ('valid_before', None),
            ('constraints', SshCertConstraintVector([])),
            ('nonce', b'\x00\x01\x02\x03'),
            ('reserved', b''),
            ('signature_key', SshHostKeyRSA(
                host_key_algorithm=SshHostKeyAlgorithm.SSH_RSA,
                public_key=PublicKey.from_params(PublicKeyParamsRsa(
                    public_exponent=0x010001,
                    modulus=0x01010203,
                )),
            )),
            ('signature', SshCertSignature(
                signature_type=SshHostKeyAlgorithm.SSH_RSA,
                signature_data=b'\x00\x01\x02\x03'
            ))
        ]))


class TestHostCertificateV01RSA(TestHostCertificateRSABase):
    def setUp(self):
        super().setUp()
        self.host_cert_bytes = bytes(
            b'\x00\x00\x00\x1c' +
            b'ssh-rsa-cert-v01@openssh.com' +
            b'\x00\x00\x00\x04' +  # nonce
            b'\x00\x01\x02\x03' +
            b'\x00\x00\x00\x01' +  # e
            b'\x03' +
            b'\x00\x00\x00\x04' +  # n
            b'\x01\x01\x02\x03' +
            b'\x01\x02\x03\x04\x05\x06\x07\x08' +  # serial
            b'\x00\x00\x00\x02' +  # certificate_type (SshCertType.SSH_CERT_TYPE_HOST)
            b'\x00\x00\x00\x08' +  # key_id
            b'\x00\x01\x02\x03\x04\x05\x06\x07' +
            b'\x00\x00\x00\x00' +  # valid_principals
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +  # valid_after
            b'\xff\xff\xff\xff\xff\xff\xff\xff' +  # valid_before
            b'\x00\x00\x00\x00' +  # critical_options
            b'\x00\x00\x00\x00' +  # extensions
            b'\x00\x00\x00\x00' +  # reserved
            b'\x00\x00\x00\x1a' +  # signature_key
            self.host_key_bytes +
            b''
        )
        self.host_cert = SshHostCertificateV01RSA(
            host_key_algorithm=SshHostKeyAlgorithm.SSH_RSA_CERT_V01_OPENSSH_COM,
            nonce=b'\x00\x01\x02\x03',
            public_key=PublicKey.from_params(PublicKeyParamsRsa(
                public_exponent=0x03,
                modulus=0x01010203,
            )),
            serial=0x0102030405060708,
            certificate_type=SshCertType.SSH_CERT_TYPE_HOST,
            key_id='\x00\x01\x02\x03\x04\x05\x06\x07',
            valid_principals=SshCertValidPrincipals([]),
            valid_after=datetime.datetime.fromtimestamp(0, datetime.timezone.utc),
            valid_before=None,
            critical_options=SshCertCriticalOptionVector([]),
            extensions=SshCertExtensionVector([]),
            reserved=b'',
            signature_key=SshHostKeyRSA(
                SshHostKeyAlgorithm.SSH_RSA,
                PublicKey.from_params(PublicKeyParamsRsa(
                    public_exponent=0x010001,
                    modulus=0x01010203,
                ))
            ),
            signature=SshCertSignature(
                SshHostKeyAlgorithm.SSH_RSA,
                b'\x00\x01\x02\x03',
            )
        )

    def test_parse(self):
        host_cert = SshHostPublicKeyVariant.parse_exact_size(self.host_cert_bytes)

        self.assertEqual(host_cert.host_key_algorithm, self.host_cert.host_key_algorithm)
        self.assertEqual(host_cert.nonce, self.host_cert.nonce)
        self.assertEqual(host_cert.public_key.params.public_exponent, self.host_cert.public_key.params.public_exponent)
        self.assertEqual(host_cert.public_key.params.modulus, self.host_cert.public_key.params.modulus)
        self.assertEqual(host_cert.serial, self.host_cert.serial)
        self.assertEqual(host_cert.certificate_type, self.host_cert.certificate_type)
        self.assertEqual(host_cert.key_id, self.host_cert.key_id)
        self.assertEqual(host_cert.valid_principals, self.host_cert.valid_principals)
        self.assertEqual(host_cert.valid_after, self.host_cert.valid_after)
        self.assertEqual(host_cert.valid_before, self.host_cert.valid_before)
        self.assertEqual(host_cert.critical_options, self.host_cert.critical_options)
        self.assertEqual(host_cert.extensions, SshCertExtensionVector([]))
        self.assertEqual(host_cert.reserved, self.host_cert.reserved)
        self.assertEqual(host_cert.signature_key, self.host_cert.signature_key)
        self.assertEqual(host_cert.signature, self.host_cert.signature)

    def test_compose(self):
        self.assertEqual(self.host_cert.compose(), self.host_cert_bytes)
        self.assertEqual(self.host_cert.key_bytes, self.host_cert_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_cert._asdict(), OrderedDict([
            ('key_type', 'host certificate'),
            ('algorithm', Authentication.RSA),
            ('size', PublicKeySize(Authentication.RSA, 32)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:vqtvWzklbsElMSGXE0G7Gk7WDHuXzd83KOC0y0Rv9TY='),
                (Hash.SHA1, 'SHA1:9Iem7KW/rAvahTzVMTmDFg93MBk='),
                (Hash.MD5, 'MD5:54:14:e3:b7:d8:7e:fa:d0:3c:52:4b:6a:8f:c4:72:6d')
            ])),
            ('known_hosts', (
                'AAAAHHNzaC1yc2EtY2VydC12MDFAb3BlbnNzaC5jb20AAAAEAAECAwAAAAEDAAAA'
                'BAEBAgMBAgMEBQYHCAAAAAIAAAAIAAECAwQFBgcAAAAAAAAAAAAAAAD/////////'
                '/wAAAAAAAAAAAAAAAAAAABoAAAAHc3NoLXJzYQAAAAMBAAEAAAAEAQECAwAAABMA'
                'AAAHc3NoLXJzYQAAAAQAAQID'
            )),
            ('host_key_algorithm', SshHostKeyAlgorithm.SSH_RSA_CERT_V01_OPENSSH_COM),
            ('public_key', PublicKey.from_params(PublicKeyParamsRsa(
                public_exponent=0x03,
                modulus=0x01010203,
            ))),
            ('nonce', b'\x00\x01\x02\x03'),
            ('serial', 0x0102030405060708),
            ('certificate_type', SshCertType.SSH_CERT_TYPE_HOST),
            ('key_id', '\x00\x01\x02\x03\x04\x05\x06\x07'),
            ('valid_principals', SshCertValidPrincipals([])),
            ('valid_after', datetime.datetime.fromtimestamp(0, datetime.timezone.utc)),
            ('valid_before', None),
            ('critical_options', SshCertCriticalOptionVector([])),
            ('extensions', SshCertExtensionVector([])),
            ('reserved', b''),
            ('signature_key', SshHostKeyRSA(
                host_key_algorithm=SshHostKeyAlgorithm.SSH_RSA,
                public_key=PublicKey.from_params(PublicKeyParamsRsa(
                    public_exponent=0x010001,
                    modulus=0x01010203,
                )),
            )),
            ('signature', SshCertSignature(
                signature_type=SshHostKeyAlgorithm.SSH_RSA,
                signature_data=b'\x00\x01\x02\x03'
            ))
        ]))


class TestHostCertificateECDSABase(TestPublicKeyBase):
    def setUp(self):
        self.point_x_bytes = b'\x80' + (256 // 8 - 1) * b'\x00'
        self.point_y_bytes = b'\x40' + (256 // 8 - 1) * b'\x00'
        self.host_key_bytes = bytes(
            b'\x00\x00\x00\x13' +  # certificate_type
            b'ecdsa-sha2-nistp256' +
            b'\x00\x00\x00\x08' +     # curve_name_length
            b'nistp256' +             # curve_name
            b'\x00\x00\x00\x41' +     # curve_data_length
            b'\x04' +                 # curve_data
            self.point_x_bytes +
            self.point_y_bytes +
            b'\x00\x00\x00\x1f' +
            b'\x00\x00\x00\x13' +  # signature_type
            b'ecdsa-sha2-nistp256' +
            b'\x00\x00\x00\x04' +  # signature_data
            b'\x00\x01\x02\x03' +
            b''
        )


class TestHostCertificateV01ECDSA(TestHostCertificateECDSABase):
    def setUp(self):
        super().setUp()
        self.point_x_bytes = b'\x80' + (256 // 8 - 1) * b'\x00'
        self.point_y_bytes = b'\x40' + (256 // 8 - 1) * b'\x00'
        self.host_cert_bytes = bytes(
            b'\x00\x00\x00\x28' +
            b'ecdsa-sha2-nistp256-cert-v01@openssh.com' +
            b'\x00\x00\x00\x04' +  # nonce
            b'\x00\x01\x02\x03' +
            b'\x00\x00\x00\x08' +     # curve_name_length
            b'nistp256' +             # curve_name
            b'\x00\x00\x00\x41' +     # curve_data_length
            b'\x04' +                 # curve_data
            self.point_x_bytes +
            self.point_y_bytes +
            b'\x01\x02\x03\x04\x05\x06\x07\x08' +  # serial
            b'\x00\x00\x00\x02' +  # certificate_type (SshCertType.SSH_CERT_TYPE_HOST)
            b'\x00\x00\x00\x08' +  # key_id
            b'\x00\x01\x02\x03\x04\x05\x06\x07' +
            b'\x00\x00\x00\x00' +  # valid_principals
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +  # valid_after
            b'\xff\xff\xff\xff\xff\xff\xff\xff' +  # valid_before
            b'\x00\x00\x00\x00' +  # critical_options
            b'\x00\x00\x00\x00' +  # extensions
            b'\x00\x00\x00\x00' +  # reserved
            b'\x00\x00\x00\x68' +  # signature_key
            self.host_key_bytes +
            b''
        )
        self.host_cert = SshHostCertificateV01ECDSA(
            host_key_algorithm=SshHostKeyAlgorithm.ECDSA_SHA2_NISTP256_CERT_V01_OPENSSH_COM,
            nonce=b'\x00\x01\x02\x03',
            public_key=PublicKey.from_params(PublicKeyParamsEcdsa(
                named_group=NamedGroup.PRIME256V1,
                point_x=2 ** 255,
                point_y=2 ** 254,
            )),
            serial=0x0102030405060708,
            certificate_type=SshCertType.SSH_CERT_TYPE_HOST,
            key_id='\x00\x01\x02\x03\x04\x05\x06\x07',
            valid_principals=SshCertValidPrincipals([]),
            valid_after=datetime.datetime.fromtimestamp(0, datetime.timezone.utc),
            valid_before=None,
            critical_options=SshCertCriticalOptionVector([]),
            extensions=SshCertExtensionVector([]),
            reserved=b'',
            signature_key=SshHostKeyECDSA(
                host_key_algorithm=SshHostKeyAlgorithm.ECDSA_SHA2_NISTP256,
                public_key=PublicKey.from_params(PublicKeyParamsEcdsa(
                    named_group=NamedGroup.PRIME256V1,
                    point_x=2 ** 255,
                    point_y=2 ** 254,
                )),
            ),
            signature=SshCertSignature(
                SshHostKeyAlgorithm.ECDSA_SHA2_NISTP256,
                b'\x00\x01\x02\x03',
            )
        )

    def test_parse(self):
        host_cert = SshHostPublicKeyVariant.parse_exact_size(self.host_cert_bytes)

        self.assertEqual(host_cert.host_key_algorithm, self.host_cert.host_key_algorithm)
        self.assertEqual(host_cert.nonce, self.host_cert.nonce)
        self.assertEqual(host_cert.public_key.params.named_group, NamedGroup.PRIME256V1)
        self.assertEqual(host_cert.public_key.params.point_x, 2 ** 255)
        self.assertEqual(host_cert.public_key.params.point_y, 2 ** 254)
        self.assertEqual(host_cert.serial, self.host_cert.serial)
        self.assertEqual(host_cert.certificate_type, self.host_cert.certificate_type)
        self.assertEqual(host_cert.key_id, self.host_cert.key_id)
        self.assertEqual(host_cert.valid_principals, self.host_cert.valid_principals)
        self.assertEqual(host_cert.valid_after, self.host_cert.valid_after)
        self.assertEqual(host_cert.valid_before, self.host_cert.valid_before)
        self.assertEqual(host_cert.critical_options, self.host_cert.critical_options)
        self.assertEqual(host_cert.extensions, SshCertExtensionVector([]))
        self.assertEqual(host_cert.reserved, self.host_cert.reserved)
        self.assertEqual(host_cert.signature_key, self.host_cert.signature_key)
        self.assertEqual(host_cert.signature, self.host_cert.signature)

    def test_compose(self):
        self.assertEqual(self.host_cert.compose(), self.host_cert_bytes)
        self.assertEqual(self.host_cert.key_bytes, self.host_cert_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_cert._asdict(), OrderedDict([
            ('key_type', 'host certificate'),
            ('algorithm', Authentication.ECDSA),
            ('size', PublicKeySize(Authentication.ECDSA, 256)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:aRBBmsAqpnH3wrOh35fa//LB/JGcuTUAPuRPEj9RWRQ='),
                (Hash.SHA1, 'SHA1:9E5qN+JPZBX4ZBfKEou/WNWtWR8='),
                (Hash.MD5, 'MD5:e9:dd:20:42:a4:36:6b:90:60:29:f8:69:5f:be:e7:13')
            ])),
            ('known_hosts', (
                'AAAAKGVjZHNhLXNoYTItbmlzdHAyNTYtY2VydC12MDFAb3BlbnNzaC5jb20AAAAE'
                'AAECAwAAAAhuaXN0cDI1NgAAAEEEgAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                'AAAAAABAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAECAwQFBgcIAAAA'
                'AgAAAAgAAQIDBAUGBwAAAAAAAAAAAAAAAP//////////AAAAAAAAAAAAAAAAAAAA'
                'aAAAABNlY2RzYS1zaGEyLW5pc3RwMjU2AAAACG5pc3RwMjU2AAAAQQSAAAAAAAAA'
                'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAEAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                'AAAAAAAAAAAAAAAAHwAAABNlY2RzYS1zaGEyLW5pc3RwMjU2AAAABAABAgM='
            )),
            ('host_key_algorithm', SshHostKeyAlgorithm.ECDSA_SHA2_NISTP256_CERT_V01_OPENSSH_COM),

            ('public_key', PublicKey.from_params(PublicKeyParamsEcdsa(
                named_group=NamedGroup.PRIME256V1,
                point_x=2 ** 255,
                point_y=2 ** 254,
            ))),
            ('nonce', b'\x00\x01\x02\x03'),
            ('serial', 0x0102030405060708),
            ('certificate_type', SshCertType.SSH_CERT_TYPE_HOST),
            ('key_id', '\x00\x01\x02\x03\x04\x05\x06\x07'),
            ('valid_principals', SshCertValidPrincipals([])),
            ('valid_after', datetime.datetime.fromtimestamp(0, datetime.timezone.utc)),
            ('valid_before', None),
            ('critical_options', SshCertCriticalOptionVector([])),
            ('extensions', SshCertExtensionVector([])),
            ('reserved', b''),
            ('signature_key', SshHostKeyECDSA(
                host_key_algorithm=SshHostKeyAlgorithm.ECDSA_SHA2_NISTP256,
                public_key=PublicKey.from_params(PublicKeyParamsEcdsa(
                    named_group=NamedGroup.PRIME256V1,
                    point_x=2 ** 255,
                    point_y=2 ** 254,
                )),
            )),
            ('signature', SshCertSignature(
                signature_type=SshHostKeyAlgorithm.ECDSA_SHA2_NISTP256,
                signature_data=b'\x00\x01\x02\x03'
            ))
        ]))


class TestHostCertificateEDDSABase(TestPublicKeyBase):
    def setUp(self):
        self.host_key_bytes = bytes(
            b'\x00\x00\x00\x17' +  # certificate_type
            b'\x00\x00\x00\x0b' +  # host_key_algorithm
            b'ssh-ed25519' +
            b'\x00\x00\x00\x04' +     # key_data_length
            b'\x00\x01\x02\x03' +     # key_data
            b'\x00\x00\x00\x17' +
            b'\x00\x00\x00\x0b' +  # signature_type
            b'ssh-ed25519' +
            b'\x00\x00\x00\x04' +  # signature_data
            b'\x00\x01\x02\x03' +
            b''
        )


class TestHostCertificateV01EDDSA(TestHostCertificateEDDSABase):
    def setUp(self):
        super().setUp()
        self.host_cert_bytes = bytes(
            b'\x00\x00\x00\x20' +
            b'ssh-ed25519-cert-v01@openssh.com' +
            b'\x00\x00\x00\x04' +  # nonce
            b'\x00\x01\x02\x03' +
            b'\x00\x00\x00\x04' +     # key_data_length
            b'\x00\x01\x02\x03' +     # key_data
            b'\x01\x02\x03\x04\x05\x06\x07\x08' +  # serial
            b'\x00\x00\x00\x02' +  # certificate_type (SshCertType.SSH_CERT_TYPE_HOST)
            b'\x00\x00\x00\x08' +  # key_id
            b'\x00\x01\x02\x03\x04\x05\x06\x07' +
            b'\x00\x00\x00\x00' +  # valid_principals
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +  # valid_after
            b'\xff\xff\xff\xff\xff\xff\xff\xff' +  # valid_before
            b'\x00\x00\x00\x00' +  # critical_options
            b'\x00\x00\x00\x00' +  # extensions
            b'\x00\x00\x00\x00' +  # reserved
            self.host_key_bytes +
            b''
        )
        self.host_cert = SshHostCertificateV01EDDSA(
            host_key_algorithm=SshHostKeyAlgorithm.SSH_ED25519_CERT_V01_OPENSSH_COM,
            nonce=b'\x00\x01\x02\x03',
            public_key=PublicKey.from_params(PublicKeyParamsEddsa(
                curve_type=NamedGroup.CURVE25519,
                key_data=b'\x00\x01\x02\x03',
            )),
            serial=0x0102030405060708,
            certificate_type=SshCertType.SSH_CERT_TYPE_HOST,
            key_id='\x00\x01\x02\x03\x04\x05\x06\x07',
            valid_principals=SshCertValidPrincipals([]),
            valid_after=datetime.datetime.fromtimestamp(0, datetime.timezone.utc),
            valid_before=None,
            critical_options=SshCertCriticalOptionVector([]),
            extensions=SshCertExtensionVector([]),
            reserved=b'',
            signature_key=SshHostKeyEDDSA(
                host_key_algorithm=SshHostKeyAlgorithm.SSH_ED25519,
                public_key=PublicKey.from_params(PublicKeyParamsEddsa(
                    curve_type=NamedGroup.CURVE25519,
                    key_data=b'\x00\x01\x02\x03',
                )),
            ),
            signature=SshCertSignature(
                SshHostKeyAlgorithm.SSH_ED25519,
                b'\x00\x01\x02\x03',
            )
        )

    def test_parse(self):
        host_cert = SshHostPublicKeyVariant.parse_exact_size(self.host_cert_bytes)

        self.assertEqual(host_cert.host_key_algorithm, self.host_cert.host_key_algorithm)
        self.assertEqual(host_cert.nonce, self.host_cert.nonce)
        self.assertEqual(host_cert.public_key.params.key_data, b'\x00\x01\x02\x03')
        self.assertEqual(host_cert.serial, self.host_cert.serial)
        self.assertEqual(host_cert.certificate_type, self.host_cert.certificate_type)
        self.assertEqual(host_cert.key_id, self.host_cert.key_id)
        self.assertEqual(host_cert.valid_principals, self.host_cert.valid_principals)
        self.assertEqual(host_cert.valid_after, self.host_cert.valid_after)
        self.assertEqual(host_cert.valid_before, self.host_cert.valid_before)
        self.assertEqual(host_cert.critical_options, self.host_cert.critical_options)
        self.assertEqual(host_cert.extensions, SshCertExtensionVector([]))
        self.assertEqual(host_cert.reserved, self.host_cert.reserved)
        self.assertEqual(host_cert.signature_key, self.host_cert.signature_key)
        self.assertEqual(host_cert.signature, self.host_cert.signature)

    def test_compose(self):
        self.assertEqual(self.host_cert.compose(), self.host_cert_bytes)
        self.assertEqual(self.host_cert.key_bytes, self.host_cert_bytes)

    def test_asdict(self):
        self.assertEqual(self.host_cert._asdict(), OrderedDict([
            ('key_type', 'host certificate'),
            ('algorithm', Authentication.EDDSA),
            ('size', PublicKeySize(Authentication.EDDSA, 256)),
            ('fingerprints', OrderedDict([
                (Hash.SHA2_256, 'SHA256:IDjjwI5W2lkjfR/gnU0pvSw6E340LushvP/N9A1HrWg='),
                (Hash.SHA1, 'SHA1:EXpev7tY2XCP2R0G6eqcqNgBpOc='),
                (Hash.MD5, 'MD5:74:06:06:3f:aa:1f:a8:34:1f:1e:6c:16:26:4c:fd:6e')
            ])),
            ('known_hosts', (
                'AAAAIHNzaC1lZDI1NTE5LWNlcnQtdjAxQG9wZW5zc2guY29tAAAABAABAgMAAAAE'
                'AAECAwECAwQFBgcIAAAAAgAAAAgAAQIDBAUGBwAAAAAAAAAAAAAAAP//////////'
                'AAAAAAAAAAAAAAAAAAAAFwAAAAtzc2gtZWQyNTUxOQAAAAQAAQIDAAAAFwAAAAtz'
                'c2gtZWQyNTUxOQAAAAQAAQID'
            )),
            ('host_key_algorithm', SshHostKeyAlgorithm.SSH_ED25519_CERT_V01_OPENSSH_COM),
            ('public_key', PublicKey.from_params(PublicKeyParamsEddsa(
                curve_type=NamedGroup.CURVE25519,
                key_data=b'\x00\x01\x02\x03',
            ))),
            ('nonce', b'\x00\x01\x02\x03'),
            ('serial', 0x0102030405060708),
            ('certificate_type', SshCertType.SSH_CERT_TYPE_HOST),
            ('key_id', '\x00\x01\x02\x03\x04\x05\x06\x07'),
            ('valid_principals', SshCertValidPrincipals([])),
            ('valid_after', datetime.datetime.fromtimestamp(0, datetime.timezone.utc)),
            ('valid_before', None),
            ('critical_options', SshCertCriticalOptionVector([])),
            ('extensions', SshCertExtensionVector([])),
            ('reserved', b''),
            ('signature_key', SshHostKeyEDDSA(
                host_key_algorithm=SshHostKeyAlgorithm.SSH_ED25519,
                public_key=PublicKey.from_params(PublicKeyParamsEddsa(
                    curve_type=NamedGroup.CURVE25519,
                    key_data=b'\x00\x01\x02\x03',
                )),
            )),
            ('signature', SshCertSignature(
                signature_type=SshHostKeyAlgorithm.SSH_ED25519,
                signature_data=b'\x00\x01\x02\x03'
            ))
        ]))


class TestX509CertificateChain(TestClasses.TestKeyBase):
    def setUp(self):
        super().setUp()

        x509_certificate = self._get_public_key_x509('snakeoil_cert.pem')
        self.x509_certificate_bytes = x509_certificate.der

        self.x509v3_ssh_rsa_certificate_bytes = bytes(
            b'\x00\x00\x00\x0e' +  # host_key_algorithm
            b'x509v3-ssh-rsa' +
            b'\x00\x00\x00\x01' +  # certificate_count
            b'\x00\x00\x03\xc4' +  # certificate_length
            self.x509_certificate_bytes +
            b'\x00\x00\x00\x01' +  # ocsp_response_count
            b'\x00\x00\x00\x04' +  # ocsp_response_length
            b'\x00\x01\x02\x03' +
            b''
        )

        self.x509v3_ssh_rsa_certificate = SshX509CertificateChain(
            SshHostKeyAlgorithm.X509V3_SSH_RSA, x509_certificate, [], [b'\x00\x01\x02\x03']
        )

    def test_key_bytes(self):
        self.assertEqual(
            self.x509v3_ssh_rsa_certificate.key_bytes,
            self.x509v3_ssh_rsa_certificate.public_key.key_bytes
        )

    def test_asdict(self):
        dict_result = self.x509v3_ssh_rsa_certificate._asdict()
        self.assertEqual(list(dict_result.keys())[-2:], ['key_type', 'certificate_chain'])
        self.assertEqual(dict_result.pop('key_type'), 'X.509 certificate chain')
        self.assertEqual(dict_result.pop('certificate_chain'), [self.x509v3_ssh_rsa_certificate.public_key])

    def test_parse(self):
        x509_certificate = SshX509CertificateChain.parse_exact_size(self.x509v3_ssh_rsa_certificate_bytes)
        self.assertEqual(x509_certificate.host_key_algorithm, SshHostKeyAlgorithm.X509V3_SSH_RSA)
        self.assertEqual(x509_certificate.public_key, self.x509v3_ssh_rsa_certificate.public_key)
        self.assertEqual(x509_certificate.issuer_certificates, [])
        self.assertEqual(x509_certificate.ocsp_responses, [b'\x00\x01\x02\x03'])

    def test_compose(self):
        self.assertEqual(self.x509v3_ssh_rsa_certificate.compose(), self.x509v3_ssh_rsa_certificate_bytes)


class TestX509Certificate(TestClasses.TestKeyBase):
    def setUp(self):
        super().setUp()

        self.x509v3_sign_rsa_header = bytes(
            b'\x00\x00\x00\x14' +  # host_key_algorithm
            b'x509v3-sign-rsa-sha1' +
            b'\x00\x00\x03\xc4' +  # public_key_length
            b''
        )

        self.x509_certificate = self._get_public_key_x509('snakeoil_cert.pem')
        self.x509_certificate_bytes = self.x509_certificate.der

        self.x509v3_sign_rsa_certificate = SshX509Certificate(
            SshHostKeyAlgorithm.X509V3_SIGN_RSA, self.x509_certificate
        )
        self.x509v3_sign_rsa_sha1_certificate = SshX509Certificate(
            SshHostKeyAlgorithm.X509V3_SIGN_RSA_SHA1, self.x509_certificate
        )

    def test_error_invalid_type(self):
        x509_certificate_bytes = self._get_public_key_x509('ecc256.badssl.com.pem').der
        with self.assertRaises(InvalidType):
            SshX509Certificate.parse_exact_size(x509_certificate_bytes)

    def test_error_invalid_certificate_value(self):
        with self.assertRaises(InvalidValue) as context_manager:
            SshX509Certificate.parse_exact_size(self.x509v3_sign_rsa_header)
        self.assertEqual(context_manager.exception.value, b'')

    def test_parse_with_host_key_type(self):
        x509_certificate = SshX509Certificate.parse_exact_size(
            self.x509v3_sign_rsa_header + self.x509_certificate_bytes
        )
        self.assertEqual(x509_certificate.host_key_algorithm, SshHostKeyAlgorithm.X509V3_SIGN_RSA_SHA1)
        self.assertEqual(x509_certificate.public_key, self.x509v3_sign_rsa_certificate.public_key)

    def test_parse_without_host_key_type(self):
        x509_certificate = SshX509Certificate.parse_exact_size(self.x509_certificate_bytes)
        self.assertEqual(x509_certificate.host_key_algorithm, SshHostKeyAlgorithm.X509V3_SIGN_RSA)
        self.assertEqual(x509_certificate.public_key, self.x509v3_sign_rsa_certificate.public_key)

    def test_key_bytes(self):
        self.assertEqual(self.x509v3_sign_rsa_certificate.key_bytes, self.x509_certificate.public_key.key_bytes)

    def test_compose_with_host_key_type(self):
        self.assertEqual(
            self.x509v3_sign_rsa_sha1_certificate.compose(),
            self.x509v3_sign_rsa_header + self.x509_certificate_bytes
        )

    def test_compose_without_host_key_type(self):
        self.assertEqual(self.x509v3_sign_rsa_certificate.compose(), self.x509_certificate_bytes)



# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-

import unittest


class TestCasesBasesHttpHeader:
    class MinimalHeader(unittest.TestCase):
        _header_minimal = None
        _header_minimal_bytes = None
        _header_minimal_markdown = None

        def test_parse_minimal(self):
            parsed_header = self._header_minimal.parse_exact_size(self._header_minimal_bytes)
            self.assertEqual(parsed_header, self._header_minimal)

        def test_compose_minimal(self):
            self.assertEqual(self._header_minimal.compose(), self._header_minimal_bytes)

        def test_markdown(self):
            self.assertEqual(self._header_minimal.as_markdown(), self._header_minimal_markdown)

    class FullHeaderBase(unittest.TestCase):
        _header_full = None
        _header_full_bytes = None

    class FullHeader(FullHeaderBase):
        def test_parse_full(self):
            parsed_header = self._header_full.parse_exact_size(self._header_full_bytes)
            self.assertEqual(parsed_header, self._header_full)

        def test_compose_full(self):
            self.assertEqual(self._header_full.compose(), self._header_full_bytes)

    class CaseInsensitiveHeader(FullHeaderBase):
        _header_full_upper_case_bytes = None
        _header_full_lower_case_bytes = None

        def test_parse_upper_case(self):
            parsed_header = self._header_full.parse_exact_size(self._header_full_upper_case_bytes)
            self.assertEqual(parsed_header, self._header_full)

        def test_parse_lower_case(self):
            parsed_header = self._header_full.parse_exact_size(self._header_full_lower_case_bytes)
            self.assertEqual(parsed_header, self._header_full)



#!/usr/bin/env python
# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-
# pylint: disable=invalid-name

import datetime
import os
import pathlib
import sys
import urllib

sys.path.insert(0, os.path.abspath('..'))
from cryptoparser.__setup__ import (  # noqa: E402, pylint: disable=wrong-import-position
    __author__,
    __description__,
    __title__,
    __version__,
)


extensions = [
    'sphinx_sitemap',
]
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'

project = __title__
copyright = f'{datetime.datetime.now().year}, {__author__}'  # pylint: disable=redefined-builtin

if 'READTHEDOCS' in os.environ:
    version = release = os.environ['READTHEDOCS_VERSION']

    html_baseurl = os.environ['READTHEDOCS_CANONICAL_URL']

    _baseurl_parsed = urllib.parse.urlparse(html_baseurl)
    _baseurl = urllib.parse.ParseResult(_baseurl_parsed.scheme, _baseurl_parsed.netloc, '/', '', '', '').geturl()

    sitemap_url_scheme = "{link}"

    _robots_txt_lines = [
        'User-agent: *',
        '',
        'Disallow: # Allow everything',
        '',
    ]
    for lang in ('en',):
        for tag in ('latest', 'stable'):
            _robots_txt_lines.append(f'Sitemap: {_baseurl}{lang}/{tag}/sitemap.xml')

    _html_extra_dir_name = 'readthedocs'
    _html_extra_path = pathlib.Path(_html_extra_dir_name)
    _html_extra_path.mkdir(exist_ok=True)
    with open(_html_extra_path / 'robots.txt', 'w+', encoding='ascii') as _robots_txt_file:
        _robots_txt_file.write(os.linesep.join(_robots_txt_lines))

    html_extra_path = [
        _html_extra_dir_name
    ]
else:
    version = release = __version__

exclude_patterns = ['_build']

html_title = __title__ + ' — ' + __description__
html_theme = 'alabaster'
html_sidebars = {
    '**': [
        'about.html',
        'navigation.html',
        'relations.html',
        'searchbox.html',
        'donate.html',
    ]
}
html_theme_options = {
    'description': __description__,
    'fixed_sidebar': True,
}



# SPDX-License-Identifier: MPL-2.0
# -*- coding: utf-8 -*-

import unittest


from cryptoparser.common.exception import NotEnoughData

from cryptoparser.ssh.record import SshRecordInit, SshRecordKexDH, SshRecordKexDHGroup
from cryptoparser.ssh.subprotocol import SshDisconnectMessage, SshReasonCode


class TestRecord(unittest.TestCase):
    def setUp(self):
        self.test_packet = SshDisconnectMessage(
            SshReasonCode.PROTOCOL_ERROR,
            'αβγ',
            'en-US'
        )
        self.test_record = SshRecordInit(self.test_packet)
        self.test_record_bytes = bytes(
            b'\x00\x00\x00\x24' +                                 # length = 0x01020304
            b'\x0b' +                                             # padding length = 0x00
            b'\x01' +                                             # message code = DISCONNECT
            b'\x00\x00\x00\x02' +                                 # reason = PROTOCOL_ERROR
            b'\x00\x00\x00\x06' +                                 # description length = 6
            'αβγ'.encode('utf-8') +                               # description
            b'\x00\x00\x00\x05' +                                 # language length = 5
            b'en-US' +                                            # language
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +                 # padding
            b'\x00\x00\x00' +                                     # padding
            b''
        )

    def test_error(self):
        with self.assertRaises(NotEnoughData) as context_manager:
            SshRecordInit.parse_exact_size(b'')
        self.assertEqual(context_manager.exception.bytes_needed, 4)

        with self.assertRaises(NotEnoughData) as context_manager:
            SshRecordInit.parse_exact_size(
                b'\x01\x02\x03\x04'  # length = 0x01020304
            )
        self.assertEqual(context_manager.exception.bytes_needed, 0x01020304)

    def test_parse(self):
        record = SshRecordInit.parse_exact_size(self.test_record_bytes)
        self.assertEqual(record.packet.reason, SshReasonCode.PROTOCOL_ERROR)
        self.assertEqual(record.packet.description, 'αβγ')
        self.assertEqual(record.packet.language, 'en-US')

        record = SshRecordKexDH.parse_exact_size(self.test_record_bytes)
        self.assertEqual(record.packet.reason, SshReasonCode.PROTOCOL_ERROR)
        self.assertEqual(record.packet.description, 'αβγ')
        self.assertEqual(record.packet.language, 'en-US')

        record = SshRecordKexDHGroup.parse_exact_size(self.test_record_bytes)
        self.assertEqual(record.packet.reason, SshReasonCode.PROTOCOL_ERROR)
        self.assertEqual(record.packet.description, 'αβγ')
        self.assertEqual(record.packet.language, 'en-US')

    def test_compose(self):
        self.assertEqual(
            self.test_record.compose(),
            self.test_record_bytes
        )
